/* c01/gfx/primitivas.c
 * 
 */
#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"
#include <stdio.h>

#define ANCHO 800
#define ALTO 640

Uint32 rojo, verde, blanco;

/*
    Las funciones ____Color de SDL_gfx
    requieren el color en un entero de 32 bis
    as: 0xRRGGBBAA, no acepta el color
    en ningn otro formato.
    
    Adems, utiliza las transparencias por defecto,
    siempre hay que especificar la opacidad del color.
*/
Uint32 color(Uint8 r, Uint8 g, Uint8 b){
    return 
        r << 24 |
        g << 16 |
        b << 8  |
        255;    //este valor es la opacidad del color
                //y debe ser mxima para que sea slido
}


/*
    Ver todas las funciones disponibles
    en el archivo de cabecera:
    /usr/include/SDL/SDL_gfxPrimitives.h
    
    Hay unas ____Color y otras _____RGBA.
*/
void dibujar(SDL_Surface *pantalla){
    //borrar la pantalla
    SDL_FillRect (pantalla, NULL, SDL_MapRGB (pantalla->format,0,0,0));
    /*Las operaciones fuera de SDL_gfx deben ser tratadas con 'SDL_MapRGB' y 'SDL_MapRGBA',
    no con 'Uint32 color(Uint8,Uint8,Uint8)'*/
    
    //dibujar un rectngulo rojo y otro verde
    boxColor(pantalla, 0,0,ANCHO/2, ALTO/2, rojo);
    boxColor(pantalla, ANCHO/2, ALTO/2, ANCHO, ALTO, verde);

    //lineas (notar la diferencia entre ambas):
    lineColor(pantalla, 0,0,ANCHO, ALTO , blanco);
    aalineColor(pantalla, 0,ALTO,ANCHO, 0 , blanco);

    //lnea horizontal amarilla    
    hlineColor(pantalla, 0, ANCHO, ALTO/2, color(255, 255, 0));
    
    stringColor (pantalla, ANCHO/2,ALTO/2,"HOLA", color(255,0,255));
    stringRGBA (pantalla, 3*ANCHO/4,ALTO/4,"BIENVENIDOS A SDL_gfx", 255,255,255,255);
}

int main(int argc, char *argv[]){
    SDL_Surface *pantalla = NULL;
    SDL_Event evento;
    const SDL_VideoInfo *info;
    int corriendo = 1;
    int profundidad_color;
    
    if (SDL_Init(SDL_INIT_VIDEO) < 0){
        printf("Error al iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);
    
    //Este if es importante para poder usar SDL_gfx
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
    } else {
        profundidad_color = 16;
    }
    
    pantalla = SDL_SetVideoMode(ANCHO, ALTO, profundidad_color,  SDL_SWSURFACE);
    if (pantalla == NULL){
        printf("Error al inicializar el modo de video: '%s'\n", SDL_GetError());
        exit(2);
    }
    
    rojo = color(255,0,0);
    verde = color(0,255,0);
    //verde = SDL_MapRGBA (pantalla->format,0,255,0, 128);  //as no funciona
    blanco = color(255, 255, 255);
    
    SDL_WM_SetCaption("Ejemplo de gfx", NULL);
    
    //hacer primer dibujo
    dibujar(pantalla);
        
    //volcar el buffer en la pantalla
    SDL_Flip(pantalla);
    
    //esperar que se cierre la ventana
    while(corriendo) {
        while(SDL_PollEvent(&evento)) {
            switch(evento.type){
                
                case SDL_QUIT:
                    corriendo = 0;
                break;
            }
        }
    }
    
    SDL_Quit();
    return 0;
}
