/* c04/campos vectoriales/main.c
 * */
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <stdio.h>
#include <math.h>
#include "colores_.h"
#include "campo_.h"
#include "escala_.h"

#define MAX_CARGAS 20
#define CARGA_INICIAL 1.0
#define INCREMENTO_CARGA 1.0
#define MIN_CANT_CUADROS 5

struct configuracion{
    int numCuadrosH;
    int numCuadrosV;
    
    Uint32 colorFondo;
    Uint32 colorFlecha;
    Uint32 colorPuntaFlecha;
    Uint32 colorCargaPositiva;
    Uint32 colorCargaNegativa;
    
    CargaElectrica cargas[MAX_CARGAS];
    int cantidadActualCargas;
    
    Escala e;
    double radioCarga; //en el marco virtual
} conf;

int profundidad_color;
const SDL_VideoInfo *info;

void configurar(void){
    conf.numCuadrosH = conf.numCuadrosV = 30;
    conf.colorFondo = BLANCO;
    conf.colorFlecha = VERDE;
    conf.colorPuntaFlecha = ROJO|AZUL;
    conf.colorCargaPositiva = ROJO;
    conf.colorCargaNegativa = NEGRO;
    conf.e.AnchoR = 600;
    conf.e.AltoR = 600;
    conf.e.XminV = conf.e.YminV = -10.0;
    conf.e.XmaxV = conf.e.YmaxV = 10.0;
    conf.radioCarga = 1.0;
    conf.cantidadActualCargas = 0;
}

//Dibuja los campos en el buffer, no en la pantalla directamente
void actualizar(SDL_Surface *pantalla){
    int i,j;
    Vector campo;
    Vector punto;
    
    borrarPantalla(pantalla, conf.colorFondo);
    
    //dibujar las cargas
    for(i=0;i<conf.cantidadActualCargas; i++){
        filledEllipseColor(pantalla,
        tR_Vx(conf.cargas[i].posicion.x,&conf.e),
        tR_Vy(conf.cargas[i].posicion.y,&conf.e),
        abs(mR_Vx(conf.radioCarga,&conf.e)),
        abs(mR_Vy(conf.radioCarga,&conf.e)), 
        (conf.cargas[i].valor>0.0)? conf.colorCargaPositiva: conf.colorCargaNegativa);
    }
    
    //dibujar el campo
    if(conf.cantidadActualCargas>0)
    for(i=0; i<conf.numCuadrosV; i++){

        //el centro de cada cuadrito
        punto.y = tV_Ry((conf.e.AltoR/conf.numCuadrosV)*(2*i+1)/2, &conf.e);

        for(j=0;j<conf.numCuadrosH;j++){
            punto.x = tV_Rx((conf.e.AnchoR/conf.numCuadrosH)*(2*j+1)/2, &conf.e);

            campoElectricoCargas(conf.cargas, conf.cantidadActualCargas, &punto, &campo);

            //dibujar las lneas
            aalineColor(pantalla, 
            tR_Vx(punto.x, &conf.e),
            tR_Vy(punto.y, &conf.e),
            //**********
            tR_Vx(punto.x, &conf.e) + (int)((campo.x/magnitudVector(&campo))*(conf.e.AnchoR/conf.numCuadrosH -1)),
            tR_Vy(punto.y, &conf.e) - (int)((campo.y/magnitudVector(&campo))*(conf.e.AltoR /conf.numCuadrosV -1)),
            conf.colorFlecha);
            
            //dibujar puntas
            filledCircleColor(pantalla, 
            tR_Vx(punto.x, &conf.e) + (int)((campo.x/magnitudVector(&campo))*(conf.e.AnchoR/conf.numCuadrosH -1)),
            tR_Vy(punto.y, &conf.e) - (int)((campo.y/magnitudVector(&campo))*(conf.e.AltoR /conf.numCuadrosV -1)),
            1,
            conf.colorPuntaFlecha);
        }
    }
}

void configurarVideo(SDL_Surface **pantalla){
    if (SDL_Init(SDL_INIT_VIDEO) < 0){
        printf("Error al iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);

    //Este if es importante para poder usar SDL_gfx
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
        //printf("%d\n", profundidad_color);
    } else {
        profundidad_color = 16;
    }
    
    *pantalla = SDL_SetVideoMode(conf.e.AnchoR, conf.e.AltoR, profundidad_color,  SDL_SWSURFACE|SDL_RESIZABLE);
    if (*pantalla == NULL){
        printf("Error al inicializar el modo de video: '%s'\n", SDL_GetError());
        exit(2);
    }
    
    SDL_WM_SetCaption("Simulador de campos elctricos", NULL);
}

/*
 * Dado un arreglo de cargas elctricas,
 * y una coordenada en el marco Real,
 * retorna el ndice del arreglo que se corresponde
 * con la carga seleccionada.
 * Si no coincide con ninguna, retorna -1 
 * 
 * */
int seleccionarCarga(CargaElectrica *cargas, int cantidad, int xR, int yR){
    int i;
    int x1,x2,y1,y2;
    for (i=0; i<cantidad; i++){
        x1 = tR_Vx(cargas[i].posicion.x,&conf.e) - mR_Vx(conf.radioCarga,&conf.e);
        x2 = tR_Vx(cargas[i].posicion.x,&conf.e) + mR_Vx(conf.radioCarga,&conf.e);
        y1 = tR_Vy(cargas[i].posicion.y,&conf.e) + mR_Vy(conf.radioCarga,&conf.e);
        y2 = tR_Vy(cargas[i].posicion.y,&conf.e) - mR_Vy(conf.radioCarga,&conf.e);
        if(x1 <= xR && xR <= x2 && y1 <= yR && yR <= y2)
            return i;
    }
    return -1;
}

int main(int argc, char *argv[]){
    SDL_Surface *pantalla = NULL;
    SDL_Event evento;
    
    int corriendo = 1;
    int seleccion = -1;
    
    configurar();
    configurarVideo(&pantalla);
    
    //hacer primer dibujo
    stringColor(pantalla,0,conf.e.AltoR/2,"Haga clic derecho para agregar una carga elctrica", conf.colorFlecha);
    
    //volcar el buffer en la pantalla
    SDL_Flip (pantalla);

    while(corriendo) {
    while(SDL_PollEvent(&evento)) {
        switch(evento.type){
        case SDL_VIDEORESIZE: {
            conf.e.AnchoR = evento.resize.w;
            conf.e.AltoR  = evento.resize.h;
            
            //redimensionar la pantalla, no hace falta liberar la anterior.
            pantalla = 
                    SDL_SetVideoMode(conf.e.AnchoR, conf.e.AltoR, profundidad_color, 
            SDL_SWSURFACE|SDL_RESIZABLE);
            if(pantalla == NULL){
                printf("Error al redimensionar la pantalla: '%s'\n", SDL_GetError());
                exit(1);
            }
            
            //La pantalla nueva aparece en blanco
            //o mejor dicho, en negro,
            //por lo que hay que volver a dibujar
            stringColor(pantalla, 0,conf.e.AltoR/2, "Haga clic derecho para agregar una carga elctrica", conf.colorFlecha);
            actualizar(pantalla);
            
            //vuelca el buffer en la pantalla:
            SDL_Flip (pantalla);
        }
        break;
        
        case SDL_MOUSEBUTTONDOWN:{
            //Seleccionar / evaluar campo
            if(evento.button.button == SDL_BUTTON_LEFT){
                seleccion=seleccionarCarga(conf.cargas,conf.cantidadActualCargas,evento.button.x,evento.button.y);
                if(seleccion==-1){
                    Vector punto;
                    Vector campo;
                    punto.x=tV_Rx(evento.button.x,&conf.e);
                    punto.y=tV_Ry(evento.button.y,&conf.e);
                    campoElectricoCargas(conf.cargas, conf.cantidadActualCargas, &punto, &campo);
                    printf("Campo electrico: (%f,%f), magnitud: %f\n",campo.x, campo.y, magnitudVector(&campo));
                }
            }
            
            //Agregar una carga
            else if(evento.button.button == SDL_BUTTON_RIGHT){
                if(conf.cantidadActualCargas < MAX_CARGAS){
                    conf.cargas[conf.cantidadActualCargas].valor=CARGA_INICIAL;
                    conf.cargas[conf.cantidadActualCargas].posicion.x=tV_Rx(evento.button.x,&conf.e);
                    conf.cargas[conf.cantidadActualCargas].posicion.y=tV_Ry(evento.button.y,&conf.e);
                    conf.cantidadActualCargas++;
                    actualizar(pantalla);
                    SDL_Flip(pantalla);
                }
                else
                    printf("Ya se alcanz el mximo nmero de cargas permitidas\n");
            }
            
            else if(evento.button.button == SDL_BUTTON_MIDDLE){
                int s;
                if((s=seleccionarCarga(conf.cargas,conf.cantidadActualCargas,evento.button.x,evento.button.y))>-1){
                    conf.cargas[s].valor = -conf.cargas[s].valor;
                    actualizar(pantalla);
                    SDL_Flip(pantalla);
                }
            }
            
            else if(evento.button.button == SDL_BUTTON_WHEELUP){
                int s;
                if((s=seleccionarCarga(conf.cargas,conf.cantidadActualCargas,evento.button.x,evento.button.y))>-1){
                    conf.cargas[s].valor += INCREMENTO_CARGA;
                    printf("Nuevo valor de la carga: %fC\n", conf.cargas[s].valor);
                }
                else{//alejarse un 10%
                    double incrementoAncho, incrementoAlto;
                    incrementoAncho = abs(conf.e.XmaxV-conf.e.XminV)*0.10;
                    incrementoAlto = abs(conf.e.YmaxV-conf.e.YminV)*0.10;
                    conf.e.XminV -= incrementoAncho/2;
                    conf.e.XmaxV += incrementoAncho/2;
                    conf.e.YminV -= incrementoAlto/2;
                    conf.e.YmaxV += incrementoAlto/2;
                }
                actualizar(pantalla);
                SDL_Flip(pantalla);
            }
            
            else if(evento.button.button == SDL_BUTTON_WHEELDOWN){
                int s;
                if((s=seleccionarCarga(conf.cargas,conf.cantidadActualCargas,evento.button.x,evento.button.y))>-1){
                    conf.cargas[s].valor -= INCREMENTO_CARGA;
                    printf("Nuevo valor de la carga: %fC\n", conf.cargas[s].valor);
                }
                else{//acercarse un 10%
                    double decrementoAncho, decrementoAlto;
                    decrementoAncho = abs(conf.e.XmaxV-conf.e.XminV)*0.10;
                    decrementoAlto = abs(conf.e.YmaxV-conf.e.YminV)*0.10;
                    conf.e.XminV += decrementoAncho/2;
                    conf.e.XmaxV -= decrementoAncho/2;
                    conf.e.YminV += decrementoAlto/2;
                    conf.e.YmaxV -= decrementoAlto/2;
                }
                actualizar(pantalla);
                SDL_Flip(pantalla);
            }
            
        }
        break;
        
        //asegurar que se deselecciona la carga
        case SDL_MOUSEBUTTONUP:{
            seleccion = -1;
        }
        break;
        
        //desplazar la carga seleccionada
        case SDL_MOUSEMOTION:{
            if(seleccion>-1){
                conf.cargas[seleccion].posicion.x=tV_Rx(evento.button.x,&conf.e);
                conf.cargas[seleccion].posicion.y=tV_Ry(evento.button.y,&conf.e);
                actualizar(pantalla);
                SDL_Flip(pantalla);
            }
        }
        break;
        
        case SDL_KEYDOWN:{
            if(evento.key.type == SDL_KEYDOWN && evento.key.keysym.mod & KMOD_SHIFT)
            switch(evento.key.keysym.sym){
                case SDLK_UP: {
                    conf.numCuadrosH++;
                    conf.numCuadrosV++;
                    actualizar(pantalla);
                    SDL_Flip(pantalla);
                }
                break;
                case SDLK_DOWN: {
                    if(conf.numCuadrosH>MIN_CANT_CUADROS)
                        conf.numCuadrosH--;
                    if(conf.numCuadrosV>MIN_CANT_CUADROS)
                        conf.numCuadrosV--;
                    actualizar(pantalla);
                    SDL_Flip(pantalla);
                }
                break;
            }
        }
        break;

        //cuando el usuario quiera cerrar la ventana, la aplicacin debe terminar
        case SDL_QUIT:
            corriendo = 0;
        break;
        }//fin del switch
    }//fin de while PollEvent
    }//fin de while corriendo
    
    SDL_Quit();
    return 0;
}
