/* c06/transformaciones3d/PortaldeVision.java
 * Representa el cuadro virtual de visin.
 * Es decir, representa la ventana de la aplicacin
 * a travs del cual, el usuario "ve" el universo 3D
*/
import java.awt.*;
    public class PortaldeVision implements Config3D{
        
        Matriz3d matriz = new Matriz3d();
        
        //se refieren al punto central del portal de visin
        //en coordenadas esfricas:
        float theta, phi;
        float distancia = 1f; //en el caso de la proyeccin ortogonal, es irrelevante
        
        int modo_proyeccion = ORTOGONAL;
        float magnitud_aumento = MAGNITUD_AUMENTO_INICIAL;
        
        //dimensiones del portal
        float AnchoV = 0f, AltoV = 0f;
        Dimension tamanhoR;
        
        Punto3d ojo = new Punto3d(0f, 0f, 0f);
        
        public PortaldeVision(float AnchoV, float AltoV, Dimension tamanhoR){
            this.AnchoV = AnchoV;
            this.AltoV = AltoV;
            this.tamanhoR = tamanhoR;
        }
        
        public void cambiarPortaldeVision(Objeto3DSimple objetos[], float AnchoV, float AltoV, Dimension tamanhoR){
            this.AnchoV = AnchoV;
            this.AltoV = AltoV;
            this.tamanhoR = tamanhoR;
            colocarPuntodeVision(objetos, theta, phi, distancia);
        }
        
        /*
        * Recibe el centro del portal de visin en coordenadas
        * esfricas (d, theta, phi)
        */
        public void colocarPuntodeVision(Objeto3DSimple objetos[], float theta, float phi, float distancia){
            this.theta = theta;
            this.phi = phi;
            this.distancia=distancia;
            
            matriz.identidad();

            matriz.rotarZ(- theta);
            matriz.rotarY(180 - phi);
            matriz.rotarZ(-90);
            matriz.trasladar(0, 0, distancia);
            
            //la media del ancho y alto real entre
            //la media del ancho y alto virtual, por
            //un factor de aumento
            matriz.escalar((float)(
                (tamanhoR.width + tamanhoR.height)/(AnchoV+AltoV)
                    /
                magnitud_aumento
                ));
            
            //traslamiento final:
            matriz.trasladar(tamanhoR.width/2, tamanhoR.height/2, 0);
            
            for(int i=0; i<objetos.length; i++)
                objetos[i].transformarProyeccion(matriz, this);
        }
        
        /*
        * Recibe el punto de visin en coordenadas esfricas (d, theta, phi)
        * donde la distancia est en 'distancia' (variable miembro)
        */
        public void colocarPuntodeVision(Objeto3DSimple objetos[], float theta, float phi){
            colocarPuntodeVision(objetos, theta, phi, distancia);
        }
        
        public void cambiarAumento(Objeto3DSimple objetos[], float incrementoAumento){
            magnitud_aumento += incrementoAumento;
            if(Math.abs(magnitud_aumento)<0.01){
                System.err.println("Error: Magnitud de aumento es casi cero");
                magnitud_aumento -= incrementoAumento;
            }
            colocarPuntodeVision(objetos, theta, phi, distancia);
        }
    }
