/* c06/transformaciones3d/Universo3D.java
 * Representa un universo virtual tridimensional,
 * con objetos tridimensionales.
*/
import java.awt.*;

public class Universo3D implements Config3D, Tildes{
    
    //  configuracin del grupo de objetos internamente:
    private Matriz3d m = new Matriz3d();
    
    //ventana por donde vemos el universo 3D:
    private PortaldeVision portal;
    
    //conjunto de objetos
    Objeto3DSimple [] objetos;
    
    public Universo3D(float Theta, float Phi){
        Objeto3DSimple ejes  = new Ejes(LARGOEJES);
        Objeto3DSimple cubo  = new Cubo(ANCHOCUBO_2);
        Objeto3DSimple plano = new Plano(anchoVirtual, altoVirtual);
        Objeto3DSimple ojo   = new Ojo(anchoVirtual, altoVirtual);
        
        objetos = new Objeto3DSimple[4];
        objetos[0] = ejes;
        objetos[1] = cubo;
        objetos[2] = plano;
        objetos[3] = ojo;
        
        portal = new PortaldeVision(anchoVirtual, altoVirtual, tamanhoReal);
        portal.colocarPuntodeVision(objetos, Theta, Phi);
    }
    public Universo3D(){
        this(45f, 45f);
    }
    
    public void seleccionarModoProyeccion(int modo){
        if((MODO_PROYECCION & modo) == MODO_PROYECCION)
            portal.modo_proyeccion = modo;
        else
            throw new IllegalArgumentException("Valor no permitido para modo de proyecci" + otilde + "n: " + modo);
    }
    public void cambiarPortaldeVision(Dimension tamanho){
        portal.cambiarPortaldeVision(objetos, anchoVirtual, altoVirtual, tamanho);
    }
    
    public void colocarPuntodeVision(float theta, float phi){
        portal.colocarPuntodeVision(objetos, theta, phi);
    }
    
    public void cambiarAumento(float incrementoAumento){
        portal.cambiarAumento(objetos, incrementoAumento);
    }
    
    //dibujar figuras
    public void dibujar(Graphics g){
        AristaSimple a;
        
        //g.setColor(Color.black);
        g.setColor(Color.white);
        g.fillRect(0,0, portal.tamanhoR.width, portal.tamanhoR.height);
        for(int i=0; i<objetos.length; i++){
            if(objetos[i].visible)
            for(int j=0; j<objetos[i].aristas.length; j++){
                a = objetos[i].aristas[j];
                g.setColor(a.color);
                g.drawLine(
                    (int)a.punto1.puntoProyectado.x,
                    (int)a.punto1.puntoProyectado.y,
                    (int)a.punto2.puntoProyectado.x,
                    (int)a.punto2.puntoProyectado.y);
            }
        }
    }
    
    public void visibilidad(int ixObjeto, boolean visible){
        if(ixObjeto==0){    //hacer visibles o invisibles todos los objetos, excepto los ejes
            for(int i=1;i<objetos.length; i++)
                objetos[i].visible = visible;
        }
        else
            objetos[ixObjeto].visible = visible;
    }
    
    public void configurar(int ixObjeto, int operacion, float [] parametros)
    throws IllegalArgumentException{
        m.identidad();
        switch(operacion){
            case OPERACION_TRASLACION:{
                m.trasladar(parametros[0], parametros[1], parametros[2]);
            }
            break;
            case OPERACION_ESCALAMIENTO:{
                m.escalar(parametros[0], parametros[1], parametros[2]);
            }
            break;
            case OPERACION_ESCALAMIENTO_UNIFORME:{
                m.escalar(parametros[0]);
            }
            break;
            case OPERACION_ROTACION_X:{
                m.rotarX(parametros[0]);
            }
            break;
            case OPERACION_ROTACION_Y:{
                m.rotarY(parametros[0]);
            }
            break;
            case OPERACION_ROTACION_Z:{
                m.rotarZ(parametros[0]);
            }
            break;
            default:
                throw new IllegalArgumentException("Error en los par" + atilde + "metros recibidos");
        }
        if(ixObjeto==0){    //configurar todos los objetos, excepto los ejes
            for(int i=1;i<objetos.length; i++)
                objetos[i].transformar(m);
        }
        else
            objetos[ixObjeto].transformar(m);
    }
    
}//fin de la clase Universo3D

/*
    Especificacin de los ejes
*/
class Ejes extends Objeto3DSimple{
    
    public Ejes(float largoEjes){
        int i;
        puntos = new VerticeSimple[4];
        aristas = new AristaSimple[3];
        
        for(i=0; i<puntos.length; i++)
            puntos[i] = new VerticeSimple();
        
        puntos[0].puntoReal = new Punto3d(largoEjes, 0f,        0f);        //eje x
        puntos[1].puntoReal = new Punto3d( 0f,       largoEjes, 0f);        //eje y
        puntos[2].puntoReal = new Punto3d( 0f,       0f,        largoEjes); //eje z
        
        puntos[3].puntoReal = new Punto3d( 0f, 0f, 0f); //origen
        
        for(i=0; i<aristas.length; i++)
            aristas[i] = new AristaSimple();
        
        aristas[0].color = Color.red;
        aristas[1].color = Color.green;
        aristas[2].color = Color.blue;
        
        aristas[0].punto1 = puntos[0];
        aristas[1].punto1 = puntos[1];
        aristas[2].punto1 = puntos[2];
        aristas[0].punto2 = aristas[1].punto2 = aristas[2].punto2 = puntos[3];
    }
}

/*
    Especificacin del ojo (la pirmide)
*/
class Ojo extends Objeto3DSimple{
    
    public Ojo(float ancho, float alto){
        int i;
        puntos = new VerticeSimple[4+1];
        aristas = new AristaSimple[6+4];
        
        for(i=0; i<puntos.length; i++)
            puntos[i] = new VerticeSimple();
        
        puntos[0].puntoReal = new Punto3d( ancho/2, alto/2, 0f);
        puntos[1].puntoReal = new Punto3d( ancho/2,-alto/2, 0f);
        puntos[2].puntoReal = new Punto3d(-ancho/2,-alto/2, 0f);
        puntos[3].puntoReal = new Punto3d(-ancho/2, alto/2, 0f);
        
        for(i=0; i<aristas.length; i++){
            aristas[i] = new AristaSimple();
            aristas[i].color = Color.black;
        }
        
        aristas[0].punto1 = puntos[0]; aristas[0].punto2 = puntos[1];
        aristas[1].punto1 = puntos[1]; aristas[1].punto2 = puntos[2];
        aristas[2].punto1 = puntos[2]; aristas[2].punto2 = puntos[3];
        aristas[3].punto1 = puntos[3]; aristas[3].punto2 = puntos[0];
        
        aristas[4].punto1 = puntos[0]; aristas[4].punto2 = puntos[2];
        aristas[5].punto1 = puntos[1]; aristas[5].punto2 = puntos[3];
        
        //efecto de direccin:
        puntos[4].puntoReal = new Punto3d(0f, 0f, 1f);
        
        for(i=6; i<10; i++){
            aristas[i].punto1 = puntos[i-6];
            aristas[i].punto2 = puntos[4];
        }
        this.visible = false;
    }
}

/* El Plano (el rectngulo)
 * */
class Plano extends Objeto3DSimple{
    
    public Plano(float ancho, float alto){
        int i;
        puntos = new VerticeSimple[4];
        aristas = new AristaSimple[6];
        
        for(i=0; i<puntos.length; i++)
            puntos[i] = new VerticeSimple();
        
        puntos[0].puntoReal = new Punto3d( ancho/2, alto/2, 0f);
        puntos[1].puntoReal = new Punto3d( ancho/2,-alto/2, 0f);
        puntos[2].puntoReal = new Punto3d(-ancho/2,-alto/2, 0f);
        puntos[3].puntoReal = new Punto3d(-ancho/2, alto/2, 0f);
        
        for(i=0; i<aristas.length; i++){
            aristas[i] = new AristaSimple();
            aristas[i].color = Color.black;
        }
        
        aristas[0].punto1 = puntos[0]; aristas[0].punto2 = puntos[1];
        aristas[1].punto1 = puntos[1]; aristas[1].punto2 = puntos[2];
        aristas[2].punto1 = puntos[2]; aristas[2].punto2 = puntos[3];
        aristas[3].punto1 = puntos[3]; aristas[3].punto2 = puntos[0];
        
        aristas[4].punto1 = puntos[0]; aristas[4].punto2 = puntos[2];
        aristas[5].punto1 = puntos[1]; aristas[5].punto2 = puntos[3];
    }
}

/*
    Especificacin del cubo :)
*/
class Cubo extends Objeto3DSimple{
    
    public Cubo(float anchoCubo_2){
        int i;
        puntos = new VerticeSimple[8];
        aristas = new AristaSimple[12];
        
        for(i=0; i<puntos.length; i++)
            puntos[i] = new VerticeSimple();
        
        puntos[0].puntoReal = new Punto3d( anchoCubo_2, anchoCubo_2, anchoCubo_2);
        puntos[1].puntoReal = new Punto3d( anchoCubo_2, anchoCubo_2,-anchoCubo_2);
        puntos[2].puntoReal = new Punto3d( anchoCubo_2,-anchoCubo_2,-anchoCubo_2);
        puntos[3].puntoReal = new Punto3d( anchoCubo_2,-anchoCubo_2, anchoCubo_2);
        
        puntos[4].puntoReal = new Punto3d(-anchoCubo_2, anchoCubo_2, anchoCubo_2);
        puntos[5].puntoReal = new Punto3d(-anchoCubo_2, anchoCubo_2,-anchoCubo_2);
        puntos[6].puntoReal = new Punto3d(-anchoCubo_2,-anchoCubo_2,-anchoCubo_2);
        puntos[7].puntoReal = new Punto3d(-anchoCubo_2,-anchoCubo_2, anchoCubo_2);
        
        for(i=0; i<aristas.length; i++){
            aristas[i] = new AristaSimple();
            aristas[i].color = Color.gray;
        }
        
        //en x positivo:
        aristas[0].punto1 = puntos[0]; aristas[0].punto2 = puntos[1];
        aristas[1].punto1 = puntos[1]; aristas[1].punto2 = puntos[2];
        aristas[2].punto1 = puntos[2]; aristas[2].punto2 = puntos[3];
        aristas[3].punto1 = puntos[3]; aristas[3].punto2 = puntos[0];
        
        //en x negativo:
        aristas[4].punto1 = puntos[4]; aristas[4].punto2 = puntos[5];
        aristas[5].punto1 = puntos[5]; aristas[5].punto2 = puntos[6];
        aristas[6].punto1 = puntos[6]; aristas[6].punto2 = puntos[7];
        aristas[7].punto1 = puntos[7]; aristas[7].punto2 = puntos[4];
        
        //las otras:
        aristas[ 8].punto1 = puntos[0]; aristas[ 8].punto2 = puntos[4];
        aristas[ 9].punto1 = puntos[1]; aristas[ 9].punto2 = puntos[5];
        aristas[10].punto1 = puntos[2]; aristas[10].punto2 = puntos[6];
        aristas[11].punto1 = puntos[3]; aristas[11].punto2 = puntos[7];
        
    }
}
