/* c07/transformaciones3d/Matriz3d.java
 * Almacena una matriz de transformacin
*/
import java.awt.*;
public class Matriz3d implements Config3D{ // Definir una clase de Matrices 3-D

    //contenido
    float e[][] = new float[4][4];
    
    static final int REEMPLAZAR = 0;
    static final int PRECONCAT = 1;
    static final int POSTCONCAT = 2;
    
    
//  necesitamos una transformacin inversa para el punto de iluminacin
//  y por defecto no est inicializada
    Matriz3d inv; boolean tiene_inversa = false;
    
    public Matriz3d(){           // Crear una matriz identidad
        identidad ();
    }
    
    public void imprimir(){
        System.out.println("\t"+e[0][0] + '\t' + e[0][1] + '\t' + e[0][2] + '\t' + e[0][3]);
        System.out.println("\t"+e[1][0] + '\t' + e[1][1] + '\t' + e[1][2] + '\t' + e[1][3]);
        System.out.println("\t"+e[2][0] + '\t' + e[2][1] + '\t' + e[2][2] + '\t' + e[2][3]);
        System.out.println("\t"+e[3][0] + '\t' + e[3][1] + '\t' + e[3][2] + '\t' + e[3][3] +'\n');
    }

    //convierte esta matriz en una matriz identidad de 4x4
    public void identidad(){
                  e[0][1] = e[0][2] = e[0][3] = 0.0f;
        e[1][0]           = e[1][2] = e[1][3] = 0.0f;
        e[2][0] = e[2][1]           = e[2][3] = 0.0f;
        e[3][0] = e[3][1] = e[3][2]           = 0.0f;
        
        e[0][0] = 1.0f;
        e[1][1] = 1.0f;
        e[2][2] = 1.0f;
        e[3][3] = 1.0f;
    }
    
    // Trasladar la matriz
    public void trasladar(float dx, float dy, float dz){
        Matriz3d matrizTemp = new Matriz3d();
        matrizTemp.e[0][3] = dx;
        matrizTemp.e[1][3] = dy;
        matrizTemp.e[2][3] = dz;
        this.multiplicar(matrizTemp, Matriz3d.POSTCONCAT);
        matrizTemp.identidad();
    }
    // Escalar la matriz por sx,sy,sz
    public void escalar(float sx, float sy, float sz){
        Matriz3d matrizTemp = new Matriz3d();
        matrizTemp.e[0][0] = sx;
        matrizTemp.e[1][1] = sy;
        matrizTemp.e[2][2] = sz;
        this.multiplicar(matrizTemp, Matriz3d.POSTCONCAT);
        matrizTemp.identidad();
    }

    // Escalar la matriz por un slo valor
    public void escalar(float s){
        Matriz3d matrizTemp = new Matriz3d();
        matrizTemp.e[0][0] = s;
        matrizTemp.e[1][1] = s;
        matrizTemp.e[2][2] = s;
        this.multiplicar(matrizTemp, Matriz3d.POSTCONCAT);
        matrizTemp.identidad();
    }
    
    // Rotar una matriz theta grados sobre el eje x
    public void rotarX(double theta){
        Matriz3d matrizTemp = new Matriz3d();
        theta *= aRad;    // Convertir el gulo a radianes

        double theta_cos = Math.cos(theta);
        double theta_sin = Math.sin(theta);

        matrizTemp.e[1][1] = (float)theta_cos;
        matrizTemp.e[1][2] = (float)-theta_sin;
        matrizTemp.e[2][1] = (float)theta_sin;
        matrizTemp.e[2][2] = (float)theta_cos;
        this.multiplicar(matrizTemp, Matriz3d.POSTCONCAT);
        matrizTemp.identidad();
    }
    // Rotar la matriz theta grados sobre el eje y
    public void rotarY(double theta){
        Matriz3d matrizTemp = new Matriz3d();
        theta *= aRad;                     // Convertir a  radianes
        double theta_cos = Math.cos(theta);
        double theta_sin = Math.sin(theta);

        matrizTemp.e[0][0] = (float)theta_cos;
        matrizTemp.e[0][2] = (float)theta_sin;
        matrizTemp.e[2][0] = (float)-theta_sin;
        matrizTemp.e[2][2] = (float)theta_cos;
        this.multiplicar(matrizTemp, Matriz3d.POSTCONCAT);
        matrizTemp.identidad();
    }
    // rotar la matriz theta grados sobre el eje Z
    public void rotarZ(double theta){
        Matriz3d matrizTemp = new Matriz3d();
        theta *= aRad;                       // Convertir a radianes
        double theta_cos = Math.cos(theta);
        double theta_sin = Math.sin(theta);

        matrizTemp.e[0][0] = (float)theta_cos;
        matrizTemp.e[0][1] = (float)-theta_sin;
        matrizTemp.e[1][0] = (float)theta_sin;
        matrizTemp.e[1][1] = (float)theta_cos;
        this.multiplicar(matrizTemp, Matriz3d.POSTCONCAT);
        matrizTemp.identidad();
    }
    // Concatenar (multiplicar) esta matriz con otra
    void multiplicar(Matriz3d matriz, int concat){
        Matriz3d A = null;
        Matriz3d B = null;
        if (concat == Matriz3d.PRECONCAT){
            A = matriz;
            B = this;
        } else if (concat == Matriz3d.POSTCONCAT){
            A = this;
            B = matriz;
        }
        else if (concat == Matriz3d.REEMPLAZAR){
            A = this;
            A.identidad();
            B = matriz;
        }

        //B*A:
        float n00 = A.e[0][0]*B.e[0][0] + A.e[1][0]*B.e[0][1] + A.e[2][0]*B.e[0][2];
        float n01 = A.e[0][1]*B.e[0][0] + A.e[1][1]*B.e[0][1] + A.e[2][1]*B.e[0][2];
        float n02 = A.e[0][2]*B.e[0][0] + A.e[1][2]*B.e[0][1] + A.e[2][2]*B.e[0][2];
        float n03 = A.e[0][3]*B.e[0][0] + A.e[1][3]*B.e[0][1] + A.e[2][3]*B.e[0][2] + B.e[0][3];

        float n10 = A.e[0][0]*B.e[1][0] + A.e[1][0]*B.e[1][1] + A.e[2][0]*B.e[1][2];
        float n11 = A.e[0][1]*B.e[1][0] + A.e[1][1]*B.e[1][1] + A.e[2][1]*B.e[1][2];
        float n12 = A.e[0][2]*B.e[1][0] + A.e[1][2]*B.e[1][1] + A.e[2][2]*B.e[1][2];
        float n13 = A.e[0][3]*B.e[1][0] + A.e[1][3]*B.e[1][1] + A.e[2][3]*B.e[1][2] + B.e[1][3];

        float n20 = A.e[0][0]*B.e[2][0] + A.e[1][0]*B.e[2][1] + A.e[2][0]*B.e[2][2];
        float n21 = A.e[0][1]*B.e[2][0] + A.e[1][1]*B.e[2][1] + A.e[2][1]*B.e[2][2];
        float n22 = A.e[0][2]*B.e[2][0] + A.e[1][2]*B.e[2][1] + A.e[2][2]*B.e[2][2];
        float n23 = A.e[0][3]*B.e[2][0] + A.e[1][3]*B.e[2][1] + A.e[2][3]*B.e[2][2] + B.e[2][3];
        /*
         * La ltima fila no importa, porque siempre va a ser [0 0 0 1]
         * */

        e[0][0] = n00;
        e[0][1] = n01;
        e[0][2] = n02;
        e[0][3] = n03;

        e[1][0] = n10;
        e[1][1] = n11;
        e[1][2] = n12;
        e[1][3] = n13;

        e[2][0] = n20;
        e[2][1] = n21;
        e[2][2] = n22;
        e[2][3] = n23;
    }
    //	crear una copia de esta matriz
    public Matriz3d copiar(){
        Matriz3d m = new Matriz3d();
        m.e[0][0] = e[0][0];
        m.e[0][1] = e[0][1];
        m.e[0][2] = e[0][2];
        m.e[0][3] = e[0][3];

        m.e[1][0] = e[1][0];
        m.e[1][1] = e[1][1];
        m.e[1][2] = e[1][2];
        m.e[1][3] = e[1][3];

        m.e[2][0] = e[2][0];
        m.e[2][1] = e[2][1];
        m.e[2][2] = e[2][2];
        m.e[2][3] = e[2][3];

        return m;
    }
    
    // a veces puede ser til:
    void calcularInversa(){
        float m[][] = new float[3][6], coef;
        int i, j, k;

        //copiando a registro temporal
        for(i=0; i<3; i++){
            for(j=0; j<3; j++)
                m[i][j] = e[i][j];
            for(j=3; j<6; j++)
                m[i][j] = (i==(j-3))?1:0;
        }

        // pivoteando
        for(i=0; i<3; i++){
            if((coef = m[i][i]) == 0f)
                if(i<2){/*si no es la ltima fila intercambiar 
                * con la fila siguiente, 
                * que tenga pivote distinto de cero*/
                    float aux[] = new float[6]; int h;
                    if(m[i+1][i] != 0.0f)
                        for(h=i; h<6; h++){
                            aux[h] = m[i][h];
                            m[i][h] = m[i+1][h];
                            m[i+1][h] = aux[h];
                        }
                    else if(i<1 && m[i+2][i] != 0f)
                        for(h=i; h<6; h++){
                            aux[h] = m[i][h];
                            m[i][h] = m[i+2][h];
                            m[i+2][h] = aux[h];
                        }
                    else{
                        //no tiene inversa
                        tiene_inversa = false;
                        return;
                    }
                    coef = m[i][i];
                } else{//si es la ltima fila, no hay ms posibles pivotes
                    //por lo tanto, no tiene inversa
                    tiene_inversa = false;
                    return;
                }
            // fin de verificacin de pivote vlido

            for(j=i; j<6; j++)
                //hacer 1 el pivote
                m[i][j] /= coef;

            for(k=i+1; k<(i+3); k++)
                if((coef = m[k%3][i]) != 0)
                    for(j=i; j<6; j++)//Hacer '-coef*F(k%3) + F(i) -> F(i)'
                        m[k%3][j] -= coef * m[i][j];
        }//fin de pivoteo

        //sacando del registro temporal
        inv = new Matriz3d();
        for(i=0; i<3; i++)
            for(j=0; j<3; j++)
                inv.e[i][j] = m[i][j+3];
        //la inversa existe
        tiene_inversa = true;
    }
}	//fin de clase Matriz3d
