/* c07/transformaciones3d/PortaldeVision.java
 * Representa el cuadro virtual de visin.
 * Es decir, representa la ventana de la aplicacin
 * a travs del cual, el usuario "ve" el universo 3D
 * 
 * En este caso particular, vamos a utilizar la 
 * distancia del origen virtual al centro del portal de visin
 * siempre igual que la distancia del centro de proyeccin
 * al centro del portal de visin.
*/
import java.awt.*;
    public class PortaldeVision implements Config3D{
        
        Matriz3d matriz = new Matriz3d();
        
        //se refieren al punto central del portal de visin
        //en coordenadas esfricas:
        float theta, phi;
        float distancia = DISTANCIA_INICIAL;
        
        //distancia del centro de proyeccin al plano de proyeccin
        float distanciaProyeccion = DISTANCIA_INICIAL;
        
        int modo_proyeccion = PERSPECTIVA;
        
        //dimensiones del portal
        float AnchoV, AltoV;
        Dimension tamanhoR;
        
        Punto3d ojo = new Punto3d(0f, 0f, 0f);
        
        public PortaldeVision(float AnchoV, float AltoV, Dimension tamanhoR){
            this.AnchoV = AnchoV;
            this.AltoV = AltoV;
            this.tamanhoR = tamanhoR;
        }
        
        public void cambiarPortaldeVision(Objeto3DSimple objetos[], float AnchoV, float AltoV, Dimension tamanhoR){
            this.AnchoV = AnchoV;
            this.AltoV = AltoV;
            this.tamanhoR = tamanhoR;
            colocarPuntodeVision(objetos, theta, phi, distancia);
        }
        
        /*
        * Recibe el punto de visin en coordenadas esfricas (distancia, theta, phi)
        * 
        * Asumimos que dicha distancia representa tanto la
        * distancia del origen virtual al centro del portal de visin
        * como a la distancia del centro de proyeccin
        * al centro del portal de visin.
        * Esto es por simplicidad en este cdigo
        * pero no necesariamente debe ser as siempre.
        */
        public void colocarPuntodeVision(Objeto3DSimple objetos[], float theta, float phi, float distancia){
            this.theta = theta;
            this.phi = phi;
            this.distancia = this.distanciaProyeccion = distancia;
            
            matriz.identidad();

            matriz.rotarZ(- theta);
            matriz.rotarY(180 - phi);
            matriz.rotarZ(-90);
            matriz.trasladar(0, 0, distancia);
            
            for(int i=0; i<objetos.length; i++)
                objetos[i].transformarProyeccion(matriz, this, true);

            matriz.identidad();
            //la media del ancho y alto real entre
            //la media del ancho y alto virtual, por
            //un factor de aumento
            matriz.escalar((float)(
                (tamanhoR.width + tamanhoR.height)/(AnchoV+AltoV)
                    // / 3f
                    /distancia
                ));
            
            //traslamiento final:
            matriz.trasladar(tamanhoR.width/2, tamanhoR.height/2, 0);
            
            for(int i=0; i<objetos.length; i++)
                objetos[i].transformarProyeccion2(matriz, this);
        }
        
        /*
        * Recibe el punto de visin en coordenadas esfricas (d, theta, phi)
        * donde la distancia est en 'distancia' (variable miembro)
        */
        public void colocarPuntodeVision(Objeto3DSimple objetos[], float theta, float phi){
            colocarPuntodeVision(objetos, theta, phi, distancia);
        }
        
        public void cambiarDistancia(Objeto3DSimple objetos[], float incrementoDistancia){
            distancia += incrementoDistancia;
            distanciaProyeccion += incrementoDistancia;
            if(Math.abs(distancia)<2f){//probar con <1f y acercarse bastante
                System.err.println("Error: Demasiado cerca");
                distancia -= incrementoDistancia;
                distanciaProyeccion -= incrementoDistancia;
            }
            System.err.println("distancia: "+distancia);
            colocarPuntodeVision(objetos, theta, phi, distancia);
        }
    }
