// /c09/bezier/bezier1.c
#include "bezier1_.h"

int BEZIER_calcular(segmentoBezier *sb, double t, double *x, double *y, double *z){
    if(sb == NULL || x == NULL || y==NULL || z == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    if(t<0.0 || t>1.0)
        return BEZIER_COD_ERROR_DOMINIO;
    
    double  _1_t1 = 1-t;
    double  _1_t2 = _1_t1 * _1_t1;
    double  _1_t3 = _1_t2 * _1_t1;
    double  t2 = t*t;
    double  t3 = t2*t;
    
    *x = _1_t3 * sb->x[0] + 3*t*_1_t2 * sb->x[1] + 3*t2*_1_t1 * sb->x[2] + t3 * sb->x[3];
    *y = _1_t3 * sb->y[0] + 3*t*_1_t2 * sb->y[1] + 3*t2*_1_t1 * sb->y[2] + t3 * sb->y[3];
    *z = _1_t3 * sb->z[0] + 3*t*_1_t2 * sb->z[1] + 3*t2*_1_t1 * sb->z[2] + t3 * sb->z[3];
    return BEZIER_COD_EXITO;
}

int BEZIER_calcular2d(segmentoBezier *sb, double t, double *x, double *y){
    if(sb == NULL || x == NULL || y == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    if(t<0.0 || t>1.0)
        return BEZIER_COD_ERROR_DOMINIO;
    
    double  _1_t1 = 1-t;
    double  _1_t2 = _1_t1 * _1_t1;
    double  _1_t3 = _1_t2 * _1_t1;
    double  t2 = t*t;
    double  t3 = t2*t;
    
    *x = _1_t3 * sb->x[0] + 3*t*_1_t2 * sb->x[1] + 3*t2*_1_t1 * sb->x[2] + t3 * sb->x[3];
    *y = _1_t3 * sb->y[0] + 3*t*_1_t2 * sb->y[1] + 3*t2*_1_t1 * sb->y[2] + t3 * sb->y[3];
    return BEZIER_COD_EXITO;
}

void BEZIER_imprimir(segmentoBezier *sb, FILE *f){
    int i;
    fprintf(f,"\nPuntos de control de Bzier:\n");
    fprintf(f,"i\tx_i\ty_i\tz_i\n");
    for(i=0; i<4; i++)
        fprintf(f,"%d\t%3.2g\t%3.2g\t%3.2g\n",
        i+1, sb->x[i], sb->y[i], sb->z[i]);
}
