// /c09/bezier/bezier1_.h
#include <stdio.h>

#define BEZIER_COD_EXITO 0
#define BEZIER_MSG_EXITO "xito\n"
#define BEZIER_COD_ERROR_PARAMETROS -1
#define BEZIER_MSG_ERROR_PARAMETROS "Error de parmetro(s)\n"
#define BEZIER_COD_ERROR_DOMINIO -2
#define BEZIER_MSG_ERROR_DOMINIO "Error de dominio\n"
#define BEZIER_COD_ERROR_MEMORIA -3
#define BEZIER_MSG_ERROR_MEMORIA "Error de solicitud de memoria dinmica\n"

/*
    Contiene los cuatro puntos de control
    para los segmentos de Bezier.
    
    El clculo de lo puntos internos del segmento es el siguiente:
    
    B(t) = Bx(t)i + By(t)j Bz(t)k,
        Bx(t) = (1-t)^3 * x[0] + 3t(1-t)^2 * x[1] + 3t^2(1-t) * x[2] + t^3 * x[3]
        By(t) = (1-t)^3 * y[0] + 3t(1-t)^2 * y[1] + 3t^2(1-t) * y[2] + t^3 * y[3]
        Bz(t) = (1-t)^3 * z[0] + 3t(1-t)^2 * z[1] + 3t^2(1-t) * z[2] + t^3 * z[3]
    para 0 <= t <= 1
*/
typedef struct{
    //coordenadas de los cuatro nodos
    double x[4];
    double y[4];
    double z[4];
} segmentoBezier;

/*
    Dado un valor del parmetro 't',
    devuelve los valores del punto calculado
    del segmento de Bzier.
    Asume que el segmento es tridimensional.
*/
int BEZIER_calcular(segmentoBezier *sb, double t, double *x, double *y, double *z);

/*
    Dado un valor del parmetro 't',
    devuelve los valores del punto calculado
    del segmento de Bzier.
    Esta versin asume que los puntos son coplanares en xy.
*/

int BEZIER_calcular2d(segmentoBezier *sb, double t, double *x, double *y);

/*
    Imprime en la salida especificada los valores
    almacenados en la estructura.
*/
void BEZIER_imprimir(segmentoBezier *sb, FILE *f);
