// /c09/bezier/bezier2.c
#include "bezier2_.h"
#include <stdlib.h>

void BEZIER_imprimir2(listaBezier *lista, FILE *f){
    nodoBezier *nb = NULL;
    int i,j;
    if(lista == NULL)
        return;
    fprintf(f,"\nPuntos de control de Bzier:\n");
    fprintf(f,"i\tx_i\ty_i\tz_i\n");
    
    //imprimir primer nodo
    fprintf(f,"%d\t%3.2g\t%3.2g\t%3.2g\n",
        i=1, lista->x, lista->y, lista->z);
    
    //'primero' no es un puntero
    for(nb = &(lista->primero); nb!=NULL; nb=nb->sig){
        fprintf(f, "---\n");
        for(j=0; j<3; j++)
            fprintf(f,"%d\t%3.2g\t%3.2g\t%3.2g\n", ++i, nb->x[j], nb->y[j], nb->z[j]);
    }
}

int BEZIER_crearLista(listaBezier *lista, int numNodos, double valorInicial){
    if(lista == NULL || numNodos < 1)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL, *temp2=NULL;
    
    lista->numNodos = numNodos;
    lista->x = lista->y = lista->z = 
    lista->primero.x[0] = lista->primero.x[1] = lista->primero.x[2] = 
    lista->primero.y[0] = lista->primero.y[1] = lista->primero.y[2] = valorInicial;
    lista->continuidad = 1;
    lista->primero.sig=lista->primero.ant=NULL;
    
    for(numNodos--;numNodos>0; numNodos--){
        if(!(temp = (nodoBezier*)malloc(sizeof(nodoBezier))))
            return BEZIER_COD_ERROR_MEMORIA;
        if(temp2==NULL){ //es el primero creado dinmicamente
            lista->primero.sig = temp2 = temp;
            temp->sig = NULL;
            temp->ant = &(lista->primero);
        }else{  //ya hay un nodo anterior en 'temp2'
            temp->sig = NULL;
            temp->ant = temp2;
            temp2->sig = temp;
            temp2 = temp;
        }
        temp->x[0]=temp->x[1]=temp->x[2]=
        temp->y[0]=temp->y[1]=temp->y[2]= valorInicial;
        temp = NULL;
    }
    return BEZIER_COD_EXITO;
}

int BEZIER_modificarLista2d(listaBezier *lista, double *x, double *y){
    if(lista == NULL || x == NULL || y == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    nodoBezier *temp=NULL;
    int i=0, j;
    
    lista->x = x[i];
    lista->y = y[i++];
    lista->z = 0.0;
    
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++){
        temp->x[j] = x[i];
        temp->y[j] = y[i++];
        temp->z[j] = 0.0;
    }
    if(lista->continuidad)
        BEZIER_acomodarContinua(lista);
    return BEZIER_COD_EXITO;
}

int BEZIER_modificarLista(listaBezier *lista, double *x, double *y, double *z){
    if(lista == NULL || x == NULL || y == NULL || z == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    nodoBezier *temp=NULL;
    int i=0, j;
    
    lista->x = x[i];
    lista->y = y[i];
    lista->z = z[i++];
    
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++){
        temp->x[j] = x[i];
        temp->y[j] = y[i];
        temp->z[j] = z[i++];
    }
    if(lista->continuidad)
        BEZIER_acomodarContinua(lista);
    return BEZIER_COD_EXITO;
}

void BEZIER_liberarLista(listaBezier *lista){
    if(lista == NULL)
        return;
    nodoBezier *nb = lista->primero.sig, *temp=NULL;
    while(nb){
        temp = nb->sig;
        free(nb);
        nb=temp;
    }
}

int BEZIER_calculardeLista2d(listaBezier *lista, double t, double *x, double *y){
    if(lista == NULL || x == NULL || y == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    if(t<0.0 || t>1.0)
        return BEZIER_COD_ERROR_DOMINIO;
    
    nodoBezier *temp=NULL, *nb=NULL;
    int i=0, j;
    
    //calcular el ndice del nodo 'nodoBezier'
    //con el que debe calcularse este 't'
    int ixBezier = (int) (t*lista->numNodos);
    if(ixBezier==lista->numNodos) ixBezier--;
    
    t = lista->numNodos * t - ixBezier;
    
    double  t1 = t;
    double  t2 = t*t;
    double  t3 = t2*t;
    double  _1_t1 = 1-t;
    double  _1_t2 = _1_t1 * _1_t1;
    double  _1_t3 = _1_t1 * _1_t2;

    nb = temp = &(lista->primero);
    if(ixBezier==0){    //es el primer 'nodoBezier'
        *x = _1_t3 * lista->x + 3*t1*_1_t2 * nb->x[0] + 3*t2*_1_t1 * nb->x[1] + t3 * nb->x[2];
        *y = _1_t3 * lista->y + 3*t1*_1_t2 * nb->y[0] + 3*t2*_1_t1 * nb->y[1] + t3 * nb->y[2];
    }else{  //no es el primer 'nodoBezier' con el que se calcular 't'
        i++;
        for(temp = temp->sig; temp!=NULL; temp=temp->sig)
            if(i==ixBezier){
                nb = temp;
                break;
            }
            else i++;
        //aqu se asume que siempre lo encuentra...
        *x = _1_t3 * nb->ant->x[2] + 3*t1*_1_t2 * nb->x[0] + 3*t2*_1_t1 * nb->x[1] + t3 * nb->x[2];
        *y = _1_t3 * nb->ant->y[2] + 3*t1*_1_t2 * nb->y[0] + 3*t2*_1_t1 * nb->y[1] + t3 * nb->y[2];
    }
    
    return BEZIER_COD_EXITO;
}

int BEZIER_calculardeLista(listaBezier *lista, double t, double *x, double *y, double *z){
    if(lista == NULL || x == NULL || y == NULL || z == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    if(t<0.0 || t>1.0)
        return BEZIER_COD_ERROR_DOMINIO;
    
    nodoBezier *temp=NULL, *nb=NULL;
    int i=0, j;
    
    //calcular el ndice del nodo 'nodoBezier'
    //con el que debe calcularse este 't'
    int ixBezier = (int) (t*lista->numNodos);
    if(ixBezier==lista->numNodos) ixBezier--;
    
    t = lista->numNodos * t - ixBezier;
    
    double  t1 = t;
    double  t2 = t*t;
    double  t3 = t2*t;
    double  _1_t1 = 1-t;
    double  _1_t2 = _1_t1 * _1_t1;
    double  _1_t3 = _1_t1 * _1_t2;

    nb = temp = &(lista->primero);
    if(ixBezier==0){    //es el primer 'nodoBezier'
        *x = _1_t3 * lista->x + 3*t1*_1_t2 * nb->x[0] + 3*t2*_1_t1 * nb->x[1] + t3 * nb->x[2];
        *y = _1_t3 * lista->y + 3*t1*_1_t2 * nb->y[0] + 3*t2*_1_t1 * nb->y[1] + t3 * nb->y[2];
        *z = _1_t3 * lista->z + 3*t1*_1_t2 * nb->z[0] + 3*t2*_1_t1 * nb->z[1] + t3 * nb->z[2];
    }else{  //no es el primer 'nodoBezier' con el que se calcular 't'
        i++;
        for(temp = temp->sig; temp!=NULL; temp=temp->sig)
            if(i==ixBezier){
                nb = temp;
                break;
            }
            else i++;
        //aqu se asume que siempre lo encuentra...
        *x = _1_t3 * nb->ant->x[2] + 3*t1*_1_t2 * nb->x[0] + 3*t2*_1_t1 * nb->x[1] + t3 * nb->x[2];
        *y = _1_t3 * nb->ant->y[2] + 3*t1*_1_t2 * nb->y[0] + 3*t2*_1_t1 * nb->y[1] + t3 * nb->y[2];
        *z = _1_t3 * nb->ant->z[2] + 3*t1*_1_t2 * nb->z[0] + 3*t2*_1_t1 * nb->z[1] + t3 * nb->z[2];
    }
    
    return BEZIER_COD_EXITO;
}

int BEZIER_actualizarNodo2d(listaBezier *lista, int ix, double x, double y){
    return BEZIER_actualizarNodo(lista, ix, x, y, 0.0);
}
int BEZIER_actualizarNodo(listaBezier *lista, int ix, double x, double y, double z){
    if(lista == NULL || ix < 0)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL;
    int i=0, j;
    
    if(ix==0){
        lista->x=x;
        lista->y=y;
        lista->z=z;
        return BEZIER_COD_EXITO;
    }
    i++;
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++)
        if(i==ix){
            //Mecanismo de coordinacin con los puntos adyacentes:
            if(lista->continuidad)
            switch(j){
                case 0:{
                    /*
                    Desplazar el penltimo punto
                    del nodo anterior,
                    si hay nodo anterior.
                    Esto es para garantizar la colinealidad
                    en las uniones de los segmentos de Bzier.
                    */
                    if(temp->ant){
                        temp->ant->x[1] = temp->ant->x[2] - (x-temp->ant->x[2]);
                        temp->ant->y[1] = temp->ant->y[2] - (y-temp->ant->y[2]);
                        temp->ant->z[1] = temp->ant->z[2] - (z-temp->ant->z[2]);
                    }
                }
                break;
                case 1:{
                    /*
                    Desplazar el primer punto
                    del nodo siguiente,
                    si hay nodo siguiente.
                    Esto es para garantizar la colinealidad
                    en las uniones de los segmentos de Bzier.
                    */
                    if(temp->sig){
                        temp->sig->x[0] = temp->x[2] - (x-temp->x[2]);
                        temp->sig->y[0] = temp->y[2] - (y-temp->y[2]);
                        temp->sig->z[0] = temp->z[2] - (z-temp->z[2]);
                    }
                }
                break;
                case 2:{
                    /*
                    Desplazar el punto anterior
                    y el siguiente, si es que
                    hay un siguiente.
                    */
                    if(temp->sig){
                        temp->x[1] += x - temp->x[2];
                        temp->y[1] += y - temp->y[2];
                        temp->z[1] += z - temp->z[2];
                        temp->sig->x[0] += x - temp->x[2];
                        temp->sig->y[0] += y - temp->y[2];
                        temp->sig->z[0] += z - temp->z[2];
                    }
                }
                break;
            }
            temp->x[j]=x;
            temp->y[j]=y;
            temp->z[j]=z;
            return BEZIER_COD_EXITO;
        }
        else i++;
    return BEZIER_COD_ERROR_DOMINIO;
}

int BEZIER_recuperarNodo2d(listaBezier *lista, int ix, double *x, double *y){
    if(lista == NULL || ix < 0 || x == NULL || y == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL;
    int i=0, j;
    
    if(ix==0){
        *x=lista->x;
        *y=lista->y;
        return BEZIER_COD_EXITO;
    }
    i++;
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++)
        if(i==ix){
            *x=temp->x[j];
            *y=temp->y[j];
            return BEZIER_COD_EXITO;
        }
        else i++;
    return BEZIER_COD_ERROR_DOMINIO;
}

int BEZIER_recuperarNodo(listaBezier *lista, int ix, double *x, double *y, double *z){
    if(lista == NULL || ix < 0 || x == NULL || y == NULL || z == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL;
    int i=0, j;
    
    if(ix==0){
        *x=lista->x;
        *y=lista->y;
        *z=lista->z;
        return BEZIER_COD_EXITO;
    }
    i++;
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++)
        if(i==ix){
            *x=temp->x[j];
            *y=temp->y[j];
            *z=temp->z[j];
            return BEZIER_COD_EXITO;
        }
        else i++;
    return BEZIER_COD_ERROR_DOMINIO;
}

int BEZIER_recuperarNodos(listaBezier *lista, double *x, double *y, double *z){
    if(lista == NULL || x == NULL || y == NULL || z == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL;
    int i=0, j;
    
    x[i]  =lista->x;
    y[i]  =lista->y;
    z[i++]=lista->z;
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++){
        x[i]  =temp->x[j];
        y[i]  =temp->y[j];
        z[i++]=temp->z[j];
    }
    return BEZIER_COD_EXITO;
}

int BEZIER_recuperarNodos2d(listaBezier *lista, double *x, double *y){
    if(lista == NULL || x == NULL || y == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL;
    int i=0, j;
    
    x[i]  =lista->x;
    y[i++]=lista->y;
    for(temp = &(lista->primero); temp!=NULL; temp=temp->sig)
    for(j=0; j<3; j++){
        x[i]  =temp->x[j];
        y[i++]=temp->y[j];
    }
    return BEZIER_COD_EXITO;
}

int BEZIER_acomodarContinua(listaBezier *lista){
    if(lista == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL;
    
    for(temp = (&(lista->primero))->sig; temp!=NULL; temp=temp->sig){
        /*
        Desplazar el primer punto
        del nodo actual.
        Esto es para garantizar la colinealidad
        en las uniones de los segmentos de Bzier.
        */
        temp->x[0] = temp->ant->x[2] - (temp->ant->x[1]-temp->ant->x[2]);
        temp->y[0] = temp->ant->y[2] - (temp->ant->y[1]-temp->ant->y[2]);
        temp->z[0] = temp->ant->z[2] - (temp->ant->z[1]-temp->ant->z[2]);
    }
    
    return BEZIER_COD_EXITO;
}

void BEZIER_activarContinuidad(listaBezier *lista){
    lista->continuidad = 1;
}

void BEZIER_desactivarContinuidad(listaBezier *lista){
    lista->continuidad = 0;
}


int BEZIER_agregarNodoNuevoaLista(listaBezier *lista, int valorInicial){
    if(lista == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp=NULL, *temp2=NULL;
    
    if(!(temp = (nodoBezier*)malloc(sizeof(nodoBezier))))
        return BEZIER_COD_ERROR_MEMORIA;
    for(temp2=&(lista->primero); temp2->sig; temp2=temp2->sig);
    
    temp->ant=temp->sig=NULL;
    
    temp2->sig=temp;
    temp->ant = temp2;
    lista->numNodos++;
    
    temp->x[0]=temp->x[1]=temp->x[2]=
    temp->y[0]=temp->y[1]=temp->y[2]= valorInicial;
    
    return BEZIER_COD_EXITO;
}

int BEZIER_agregarNodoaLista(listaBezier *lista, nodoBezier *nb){
    if(lista == NULL || nb == NULL)
        return BEZIER_COD_ERROR_PARAMETROS;
    
    nodoBezier *temp2=NULL;
    
    for(temp2=&(lista->primero); temp2->sig; temp2=temp2->sig);
    
    nb->ant=nb->sig=NULL;
    
    temp2->sig=nb;
    nb->ant = temp2;
    lista->numNodos++;
    
    return BEZIER_COD_EXITO;
}
