// /c09/bezier/bezier2_.h
#include "bezier1_.h"

/*********** Unin de segmentos de Bzier *************************/

/*
    Nodo para una lista lineal doble con nodo de control
    para modelar una secuencia de segmentos de Bzier
    con al menos un segmento.
    Pero el nodo de control no es del mismo tipo
    de los nodos.
*/
typedef struct nodoBezier{
    //coordenadas de los tres nodos,
    //el ltimo coincide con el primero
    //del siguiente segmento.
    double x[3];
    double y[3];
    double z[3];
    
    struct nodoBezier *ant;
    struct nodoBezier *sig;
} nodoBezier;

/*
    Nodo de control para una secuencia de
    segmentos de Bzier.
*/
typedef struct{
    //la coordenada del primer punto
    //del primer segmento:
    double x, y, z;
    
    nodoBezier primero;
    int numNodos;   //representa el nmero de estructuras, no de puntos
    int continuidad;//indica si se garantizar la continuidad de 3er orden
} listaBezier;

/*
    Imprime en la salida especificada los valores
    almacenados en la estructura.
*/
void BEZIER_imprimir2(listaBezier *lista, FILE *f);

/*
    'numNodos' representa el nmero de segmentos de Bzier deseados
*/
int BEZIER_crearLista(listaBezier *lista, int numNodos, double valorInicial);

/*
    La longitud de los arreglos de entrada 'x' y 'y'
    se asume a '3*lista->numNodos+1'.
    Tambin se asume que los puntos son coplanares en xy.
    Se asume que la 'lista' ya ha sido creada.
*/
int BEZIER_modificarLista2d(listaBezier *lista, double *x, double *y);

/*
    La longitud de los arreglos de entrada 'x', 'y' y 'z'
    se asume a '3*lista->numNodos+1'.
    Se asume que la 'lista' ya ha sido creada.
*/
int BEZIER_modificarLista(listaBezier *lista, double *x, double *y, double *z);

/*
    Libera la memoria utilizada para
    almacenar los 'nodoBezier'
*/
void BEZIER_liberarLista(listaBezier *lista);

/*
    Dado un valor del parmetro 't',
    devuelve los valores del punto calculado
    del segmento de Bzier.
    Asume que el segmento es tridimensional.
*/
int BEZIER_calculardeLista(listaBezier *lista, double t, double *x, double *y, double *z);

/*
    Dado un valor del parmetro 't',
    devuelve los valores del punto calculado
    del segmento de Bzier.
    Esta versin asume que los puntos son coplanares en xy.
*/
int BEZIER_calculardeLista2d(listaBezier *lista, double t, double *x, double *y);

/*
    Dado un ndice de punto, actualizar el nodo de Bzier correspondiente
*/
int BEZIER_actualizarNodo(listaBezier *lista, int ix, double x, double y, double z);

/*
    Dado un ndice, recuperar el punto
    en las direcciones de 'x' y 'y'.
*/
int BEZIER_recuperarNodo2d(listaBezier *lista, int ix, double *x, double *y);

/*
    Dado un ndice, recuperar el punto
    en las direcciones de 'x', 'y' y 'z'.
*/
int BEZIER_recuperarNodo(listaBezier *lista, int ix, double *x, double *y, double *z);

/*
    Copia los valores de todos los puntos a los arreglos 'x', 'y' y 'z'.
    Estos arreglos deben contener suficiente espacio.
*/
int BEZIER_recuperarNodos(listaBezier *lista, double *x, double *y, double *z);

/*
    Copia los valores de todos los puntos a los arreglos 'x' y 'y'.
    Estos arreglos deben contener suficiente espacio.
    Asume que los puntos son coplanares en xy.
*/
int BEZIER_recuperarNodos2d(listaBezier *lista, double *x, double *y);

/*
    Garantiza que la curva sea continua, forzando
    algunos puntos.
    Tendrn prioridad los puntos previos.
*/
int BEZIER_acomodarContinua(listaBezier *lista);

void BEZIER_activarContinuidad(listaBezier *lista);

void BEZIER_desactivarContinuidad(listaBezier *lista);


/*
 * Agrega un segmento nuevo de Bzier
 * al final de la Lista especificada
 * */
int BEZIER_agregarNodoNuevoaLista(listaBezier *lista, int valorInicial);

/*
 * Agrega un segmento especificado de Bzier
 * al final de la Lista especificada
 * */
int BEZIER_agregarNodoaLista(listaBezier *lista, nodoBezier *nb);
