// /c09/bezier/main1.c
#include <SDL/SDL.h>
#include "bezier1_.h"
#include <SDL/SDL_gfxPrimitives.h>

#define ANCHO 640
#define ALTO  480

#define ANCHO_RECTANGULO 15

#define XMIN  0.0
#define XMAX 15.0
#define YMIN  0.0
#define YMAX 15.0
#define TPASO 0.01

#define TAM 4

static inline int x_real(double x_virtual){
    return (int) (ANCHO * x_virtual / XMAX);
}
static inline int y_real(double y_virtual){
    return (int) (ALTO * (1.0 - y_virtual / YMAX));
}
static inline double x_virtual(int x_real){
    return (XMAX * x_real) / ANCHO;
}
static inline double y_virtual(int y_real){
    return YMAX * (1.0 - y_real / (double)ALTO);
}

Uint32 gfxColor(Uint8 r, Uint8 g, Uint8 b){
    return 
        r << 24 |
        g << 16 |
        b << 8  |
        255;    //este valor es la opacidad del color
                //y debe ser mxima para que sea slido
}

//Variables globales
double x[TAM] = {1.3, 3.0, 6.0, 13.0};
double y[TAM] = {1.3, 5.0, 7.4, 14.2};
segmentoBezier sb;
int i, j, k, l;
double tvar, xvar, yvar;
Uint32 color_fondo, color1, color2, color3;

void dibujo(SDL_Surface *pantalla){
    i=j=k=l=0;
    SDL_Rect rect;
    
    //Borra la pantalla
    SDL_FillRect(pantalla, NULL, color_fondo);
    
    for(i=1; i<4; i++){
        lineColor(pantalla,
            x_real(sb.x[i-1]), y_real(sb.y[i-1]),
            x_real(sb.x[i])  , y_real(sb.y[i])  , color3);
    }
    
    tvar = 0.0;
    BEZIER_calcular2d(&sb, tvar, &xvar, &yvar);
    i=x_real(xvar);
    j=y_real(yvar);
    for(tvar = TPASO; tvar <= 1.0; tvar+=TPASO){
        if(!BEZIER_calcular2d(&sb, tvar, &xvar, &yvar)){
            lineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color1);
            i=k; j=l;
        }
    }
    BEZIER_calcular2d(&sb, 1.0, &xvar, &yvar);
    lineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color1);
    
    //dibujar los rectngulos de control
    rect.w = rect.h = ANCHO_RECTANGULO;
    for(i=0; i<4; i++){
        rect.x = x_real(sb.x[i])-ANCHO_RECTANGULO/2;
        rect.y = y_real(sb.y[i])-ANCHO_RECTANGULO/2;
        rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color2);
    }
    
    //vuelca el buffer en la pantalla:
    SDL_Flip (pantalla);
}

int main(int argc, char *argv[]){
    SDL_Surface *pantalla = NULL;
    SDL_Event evento;
    int profundidad_color;
    const SDL_VideoInfo *info;
    Uint32 color;
    int i;
    
    int corriendo = 1;
    int seleccionado = 0;
    
    if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
        fprintf(stderr, "No se puede iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);
    
    //Este if es importante para poder usar SDL_gfx
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
        //printf("%d\n", profundidad_color);
    } else {
        profundidad_color = 16;
    }
    
    pantalla = SDL_SetVideoMode(ANCHO, ALTO, profundidad_color, SDL_SWSURFACE);
    if(pantalla == NULL)
    {
        fprintf(stderr, "No se puede establecer el modo de video %dx%d: %s\n",
            ANCHO, ALTO, SDL_GetError());
        exit(1);
    }
    SDL_WM_SetCaption("Segmento de Bezier", NULL);
    
    color_fondo = SDL_MapRGB (pantalla->format,0,0,0);
    color1 = gfxColor(255,255,255);
    color2 = gfxColor(255,0,0);
    color3 = gfxColor(0,0,255);
    
    for(i=0; i<4; i++){
        sb.x[i] = x[i];
        sb.y[i] = y[i];
    }
    //BEZIER_imprimir(&sb, stdout);
    
    dibujo(pantalla);
    
    while(corriendo) {
        while(SDL_PollEvent(&evento)) {
            switch(evento.type){
                case SDL_MOUSEMOTION:{
                    if(seleccionado){
                        //actualizar el punto
                        sb.x[i]=x_virtual(evento.button.x);
                        sb.y[i]=y_virtual(evento.button.y);
                        dibujo(pantalla);
                    }
                }
                    break;
                case SDL_MOUSEBUTTONDOWN:{
                    for(i=0; i<4; i++){
                        if(evento.button.button == SDL_BUTTON_LEFT && // si hace clic sobre un nodo...
                            ((evento.button.x > x_real(sb.x[i])-ANCHO_RECTANGULO/2) &&
                             (evento.button.y > y_real(sb.y[i])-ANCHO_RECTANGULO/2) &&
                             (evento.button.x < x_real(sb.x[i])+ANCHO_RECTANGULO/2) &&
                             (evento.button.y < y_real(sb.y[i])+ANCHO_RECTANGULO/2))
                        ){
                            //se selecciona y el ndice queda en 'i'
                            seleccionado = 1;
                            //printf("seleccionado\n");
                            break;
                        }
                    }
                }
                    break;
                case SDL_MOUSEBUTTONUP:
                    seleccionado = 0;
                    break;
                
                case SDL_KEYDOWN:
                    if(evento.key.keysym.sym==SDLK_SPACE)
                        dibujo(pantalla);
                    break;
                case SDL_QUIT:
                    corriendo = 0;
                    break;
            }
        }
    }

    SDL_Quit();
    return 0;
}
