// /c09/bezier/main2.c
#include <SDL/SDL.h>
#include "bezier2_.h"
#include <SDL/SDL_gfxPrimitives.h>

#define ANCHO 640
#define ALTO  480

#define ANCHO_RECTANGULO 15

#define XMIN  0.0
#define XMAX 15.0
#define YMIN  0.0
#define YMAX 15.0
#define TPASO 0.01

#define TAM 10
//Debe cumplirse la siguiente relacin:
//numPuntos = 3*numNodos+1;
double x[TAM] = {1.3, 3.0, 6.0, 5.0, 4.0, 5.0, 6.0, 8.0, 10.0, 9.0};
double y[TAM] = {1.3, 5.0, 7.4, 9.2, 2.0, 3.0, 4.0, 5.0, 7.0, 4.0};
listaBezier lista;


static inline int x_real(double x_virtual){
    return (int) (ANCHO * x_virtual / XMAX);
}
static inline int y_real(double y_virtual){
    return (int) (ALTO * (1.0 - y_virtual / YMAX));
}
static inline double x_virtual(int x_real){
    return (XMAX * x_real) / ANCHO;
}
static inline double y_virtual(int y_real){
    return YMAX * (1.0 - y_real / (double)ALTO);
}

Uint32 gfxColor(Uint8 r, Uint8 g, Uint8 b){
    return 
        r << 24 |
        g << 16 |
        b << 8  |
        255;    //este valor es la opacidad del color
                //y debe ser mxima para que sea slido
}

//Variables globales

int i, j, k, l;
double tvar, xvar, yvar;
Uint32 color_fondo, color_curva, color_lineas, color_union_nodos, color_union;
int mostrar_lineas, mostrar_cuadrados;

void dibujo(SDL_Surface *pantalla){
    i=j=k=l=0;
    SDL_Rect rect;
    
    //Borra la pantalla
    SDL_FillRect(pantalla, NULL, color_fondo);
    
    //dibujar los rectngulos de control
    rect.w = rect.h = ANCHO_RECTANGULO;
    if(mostrar_cuadrados)
    for(i=0; i<TAM; i++){
        rect.x = x_real(x[i])-ANCHO_RECTANGULO/2;
        rect.y = y_real(y[i])-ANCHO_RECTANGULO/2;
        if(lista.continuidad)
            switch(i%3){
                case 0:
                    rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union_nodos);
                break;
                default:
                    rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union);
                break;
            }
        else
            rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union_nodos);
    }
    
    //dibujar lneas entre los nodos
    if(mostrar_lineas)
    for(i=1; i<TAM; i++){
        lineColor(pantalla,
            x_real(x[i-1]), y_real(y[i-1]),
            x_real(x[i])  , y_real(y[i])  , color_lineas);
    }
    
    //calcular los puntos de la curva
    tvar = 0.0;
    BEZIER_calculardeLista2d(&lista, tvar, &xvar, &yvar);
    i=x_real(xvar);
    j=y_real(yvar);
    for(tvar = TPASO; tvar <= 1.0; tvar+=TPASO){
        if(!BEZIER_calculardeLista2d(&lista, tvar, &xvar, &yvar)){
            lineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color_curva);
            i=k; j=l;
        }
    }
    BEZIER_calculardeLista2d(&lista, tvar, &xvar, &yvar);
    lineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color_curva);

    //vuelca el buffer en la pantalla:
    SDL_Flip (pantalla);
}

int main(int argc, char *argv[]){
    SDL_Surface *pantalla = NULL;
    SDL_Event evento;
    int profundidad_color;
    const SDL_VideoInfo *info;
    Uint32 color;
    int i;
    
    int corriendo = 1;
    int seleccionado = 0;
    
    mostrar_cuadrados = 1;
    mostrar_lineas = 1;
    
    if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
        fprintf(stderr, "No se puede iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);
    
    //Este if es importante para poder usar SDL_gfx
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
        //printf("%d\n", profundidad_color);
    } else {
        profundidad_color = 16;
    }
    
    pantalla = SDL_SetVideoMode(ANCHO, ALTO, profundidad_color, SDL_SWSURFACE);
    if(pantalla == NULL)
    {
        fprintf(stderr, "No se puede establecer el modo de video %dx%d: %s\n",
            ANCHO, ALTO, SDL_GetError());
        exit(1);
    }
    SDL_WM_SetCaption("Segmentos de Bezier", NULL);
    
    color_fondo = SDL_MapRGB (pantalla->format,0,0,0);
    color_curva = gfxColor(255,255,255); //lnea
    color_lineas = gfxColor(0,0,255);     //unin puntos
    color_union_nodos = gfxColor(255,0,0);     //unin de nodos
    color_union = gfxColor(255,255,0);   //puntos adyacentes
    
    BEZIER_crearLista(&lista, (TAM-1)/3,0.0);
    BEZIER_modificarLista2d(&lista, x, y);
    BEZIER_recuperarNodos2d(&lista, x, y);
    BEZIER_imprimir2(&lista, stdout);
    
    dibujo(pantalla);
    
    while(corriendo) {
        while(SDL_PollEvent(&evento)) {
            switch(evento.type){
                case SDL_MOUSEMOTION:{
                    if(seleccionado){
                        //actualizar el punto
                        x[i]=x_virtual(evento.button.x);
                        y[i]=y_virtual(evento.button.y);
                        BEZIER_actualizarNodo2d(&lista, i, x[i], y[i]);
                        BEZIER_recuperarNodos2d(&lista, x, y);
                        dibujo(pantalla);
                    }
                }
                    break;
                case SDL_MOUSEBUTTONDOWN:{
                    for(i=0; i<TAM; i++){
                        if(evento.button.button == SDL_BUTTON_LEFT && // si hace clic sobre un nodo...
                            ((evento.button.x > x_real(x[i])-ANCHO_RECTANGULO/2) &&
                             (evento.button.y > y_real(y[i])-ANCHO_RECTANGULO/2) &&
                             (evento.button.x < x_real(x[i])+ANCHO_RECTANGULO/2) &&
                             (evento.button.y < y_real(y[i])+ANCHO_RECTANGULO/2))
                        ){
                            //se selecciona y el ndice queda en 'i'
                            seleccionado = 1;
                            //printf("seleccionado\n");
                            break;
                        }
                    }
                }
                    break;
                case SDL_MOUSEBUTTONUP:
                    seleccionado = 0;
                    break;
                
                case SDL_KEYDOWN:
                    switch(evento.key.keysym.sym){
                        case SDLK_i: {
                            BEZIER_imprimir2(&lista, stdout);
                        }
                        break;
                        case SDLK_l: {
                            mostrar_lineas = !mostrar_lineas;
                            dibujo(pantalla);
                        }
                        break;
                        case SDLK_c: {
                            mostrar_cuadrados = !mostrar_cuadrados;
                            dibujo(pantalla);
                        }
                        break;
                        case SDLK_r: {
                            dibujo(pantalla);
                        }
                        break;
                        case SDLK_x: {
                            lista.continuidad = !lista.continuidad;
                            if(lista.continuidad){
                                BEZIER_acomodarContinua(&lista);
                                BEZIER_recuperarNodos2d(&lista, x, y);
                            }
                            dibujo(pantalla);
                        }
                        break;
                    }
                    break;
                    
                case SDL_QUIT:
                    corriendo = 0;
                    break;
            }
        }
    }

    SDL_Quit();
    return 0;
}
