// /c09/bezier/main3.c
#include <SDL/SDL.h>
#include "bezier3_.h"
#include <SDL/SDL_gfxPrimitives.h>

#define ANCHO 800
#define ALTO  800

#define ANCHO_RECTANGULO 10

double 
XMIN = 0.0,
XMAX = 200.0,
YMIN = 0.0,
YMAX = 200.0;
#define TPASO 0.005
#define VALOR_INICIAL 100.0

listaDeListaBezier listal;


static inline int x_real(double x_virtual){
    return (int) (ANCHO * x_virtual / XMAX);
}
static inline int y_real(double y_virtual){
    return (int) (ALTO * (1.0 - y_virtual / YMAX));
}
static inline double x_virtual(int x_real){
    return (XMAX * x_real) / ANCHO;
}
static inline double y_virtual(int y_real){
    return YMAX * (1.0 - y_real / (double)ALTO);
}

Uint32 gfxColor(Uint8 r, Uint8 g, Uint8 b, Uint8 a){
    return 
        r << 24 |
        g << 16 |
        b << 8  |
        a;
}

//Variables globales

int i, j, k, l;
double tvar, xvar, yvar;
Uint32 color_fondo, color_curva, color_linea, color_union_nodos, color_union, color_primer_nodo;
int mostrar_lineas, mostrar_cuadrados;

void dibujo(SDL_Surface *pantalla){
    listaDeListaBezier *lltemp = NULL;
    listaBezier *lista = NULL;
    nodoBezier *nodoB = NULL;
    i=j=k=l=0;
    int m;
    double x, y;
    SDL_Rect rect;
    
    //Borra la pantalla
    SDL_FillRect(pantalla, NULL, color_fondo);
    
    rect.w = rect.h = ANCHO_RECTANGULO;
    
    for(lltemp = &listal; lltemp; lltemp = lltemp->sig){
        lista = &(lltemp->listaB);
        
        //dibujar los rectngulos de control
        x = lista->x; y= lista->y;
        if(mostrar_cuadrados){
            for(nodoB=&(lista->primero); nodoB; nodoB = nodoB->sig){
                for(i=0; i<3; i++){
                    rect.x = x_real(x)-ANCHO_RECTANGULO/2;
                    rect.y = y_real(y)-ANCHO_RECTANGULO/2;
                    if(lista->continuidad){
                        if(i==0)
                            rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union_nodos);
                        else
                            rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union);
                    }else
                        rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union_nodos);
                    x = nodoB->x[i];
                    y = nodoB->y[i];
                }
                //dibujar el ltimo rectngulo:
                rect.x = x_real(x)-ANCHO_RECTANGULO/2;
                rect.y = y_real(y)-ANCHO_RECTANGULO/2;
                if(lista->continuidad)
                    rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union_nodos);
                else
                    rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_union_nodos);
            }
            rect.x = x_real(lista->x)-ANCHO_RECTANGULO/2;
            rect.y = y_real(lista->y)-ANCHO_RECTANGULO/2;
            rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color_primer_nodo);
        }
        
        //dibujar lneas entre los nodos
        i=x_real(lista->x);
        j=y_real(lista->y);
        if(mostrar_lineas)
        for(nodoB=&(lista->primero); nodoB; nodoB = nodoB->sig){
            for(m=0; m<3; m++){
                aalineColor(pantalla, i, j, k=x_real(nodoB->x[m]), l=y_real(nodoB->y[m]), color_linea);
                i=k; j=l;
            }
        }
        
        //calcular los puntos de la curva
        tvar = 0.0;
        BEZIER_calculardeLista2d(lista, tvar, &xvar, &yvar);
        i=x_real(xvar);
        j=y_real(yvar);
        for(tvar = TPASO; tvar <= 1.0; tvar+=TPASO){
            if(!BEZIER_calculardeLista2d(lista, tvar, &xvar, &yvar)){
                aalineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color_curva);
                i=k; j=l;
            }
        }
        BEZIER_calculardeLista2d(lista, 1.0, &xvar, &yvar);
        aalineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color_curva);

    }
    //vuelca el buffer en la pantalla:
    SDL_Flip (pantalla);
}

int main(int argc, char *argv[]){
    SDL_Surface *pantalla = NULL;
    SDL_Event evento;
    int profundidad_color;
    const SDL_VideoInfo *info;
    Uint32 color;
    
    //ndice de seleccin de una de las listas
    int i;
    listaDeListaBezier *listaSeleccion = NULL;
    double x,y;
    
    int corriendo = 1;
    int seleccionado = 0;
    
    mostrar_cuadrados = 1;
    mostrar_lineas = 1;
    
    if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
        fprintf(stderr, "No se puede iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);
    
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
    } else {
        profundidad_color = 16;
    }
    
    pantalla = SDL_SetVideoMode(ANCHO, ALTO, profundidad_color, SDL_SWSURFACE);
    if(pantalla == NULL){
        fprintf(stderr, "No se puede establecer el modo de video %dx%d: %s\n", ANCHO, ALTO, SDL_GetError());
        exit(1);
    }
    SDL_WM_SetCaption("Varios Segmentos de Bezier", NULL);
    
    color_fondo = SDL_MapRGB (pantalla->format,255,255,255);
    color_curva = gfxColor(0,0,0,255); //lnea
    color_linea = gfxColor(128,128,128,255);     //unin puntos
    color_union_nodos = gfxColor(255,0,0,255);     //unin de nodos
    color_union = gfxColor(0,0,255,128);   //puntos adyacentes
    color_primer_nodo = gfxColor(128,128,128,255); //color del nodo cabeza de cada lista
    
    BEZIER_configurarListadeListas(&listal,VALOR_INICIAL);
    listal.listaB.x = 
    listal.listaB.y = 
    listal.listaB.z = VALOR_INICIAL;
    BEZIER_imprimir3(&listal, stdout);
    
    dibujo(pantalla);
    
    while(corriendo) {
        while(SDL_PollEvent(&evento)) {
            switch(evento.type){
                case SDL_MOUSEMOTION:{
                    if(seleccionado){
                        //actualizar el punto
                        BEZIER_actualizarNodo2d(&listaSeleccion->listaB, i, x_virtual(evento.button.x), y_virtual(evento.button.y));
                        dibujo(pantalla);
                    }
                }
                    break;
                case SDL_MOUSEBUTTONDOWN:{
                    //trasladar punto seleccionado
                    if(evento.button.button==SDL_BUTTON_LEFT){
                        seleccionado=0;
                        for(listaSeleccion = &listal; listaSeleccion; listaSeleccion = listaSeleccion->sig){
                            for(i=0; BEZIER_COD_ERROR_DOMINIO!=BEZIER_recuperarNodo2d(&listaSeleccion->listaB, i, &x, &y); i++)
                                if(
                                    (evento.button.x > x_real(x)-ANCHO_RECTANGULO/2) &&
                                    (evento.button.y > y_real(y)-ANCHO_RECTANGULO/2) &&
                                    (evento.button.x < x_real(x)+ANCHO_RECTANGULO/2) &&
                                    (evento.button.y < y_real(y)+ANCHO_RECTANGULO/2)
                                ){
                                    seleccionado = 1;
                                    //el ndice queda en 'i'
                                    //y la lista queda en listaSeleccion->listaB
                                    break;
                                }
                            if(seleccionado) break;
                        }
                    }
                    else if(evento.button.button==SDL_BUTTON_RIGHT){
                        //si hace SHIFT+clic derecho agregar nueva lista de segmentos de Bzier
                        if( SDL_GetModState() & KMOD_SHIFT ){
                            BEZIER_agregarListaaLista(&listal,VALOR_INICIAL);
                        }
                        else
                        //agregar un segmento ms a la lista seleccionada
                        //si se hace clic derecho sobre la cabeza de la lista
                        for(listaSeleccion = &listal; listaSeleccion; listaSeleccion = listaSeleccion->sig){
                            BEZIER_recuperarNodo2d(&listaSeleccion->listaB, 0, &x, &y);
                            if(
                                (evento.button.x > x_real(x)-ANCHO_RECTANGULO/2) &&
                                (evento.button.y > y_real(y)-ANCHO_RECTANGULO/2) &&
                                (evento.button.x < x_real(x)+ANCHO_RECTANGULO/2) &&
                                (evento.button.y < y_real(y)+ANCHO_RECTANGULO/2)
                            ){
                                BEZIER_agregarNodoNuevoaLista(&listaSeleccion->listaB,VALOR_INICIAL);
                                BEZIER_acomodarContinua(&listaSeleccion->listaB);
                                listaSeleccion=NULL;
                                break;
                            }
                        }
                        dibujo(pantalla);
                    }
                    // hay que corregir el problema de llegar a cero...
                    else if(evento.button.button == SDL_BUTTON_WHEELUP){
                        //alejarse un 1%
                        double incrementoAncho, incrementoAlto;
                        incrementoAncho = abs(XMAX-XMIN)*0.01;
                        incrementoAlto = abs(YMAX-YMIN)*0.01;
                        XMIN -= incrementoAncho/2;
                        XMAX += incrementoAncho/2;
                        YMIN -= incrementoAlto/2;
                        YMAX += incrementoAlto/2;
                        dibujo(pantalla);
                    }
                    
                    else if(evento.button.button == SDL_BUTTON_WHEELDOWN){
                        //acercarse un 1%
                        double decrementoAncho, decrementoAlto;
                        decrementoAncho = abs(XMAX-XMIN)*0.01;
                        decrementoAlto = abs(YMAX-YMIN)*0.01;
                        XMIN += decrementoAncho/2;
                        XMAX -= decrementoAncho/2;
                        YMIN += decrementoAlto/2;
                        YMAX -= decrementoAlto/2;
                        dibujo(pantalla);
                    }
                    
                }
                    break;
                case SDL_MOUSEBUTTONUP:
                    seleccionado = 0;
                    i=-1;
                    listaSeleccion = NULL;
                    break;
                
                case SDL_KEYDOWN:
                    switch(evento.key.keysym.sym){
                        case SDLK_i: {
                            BEZIER_imprimir3(&listal, stdout);
                        }
                        break;
                        case SDLK_l: {
                            mostrar_lineas = !mostrar_lineas;
                            dibujo(pantalla);
                        }
                        break;
                        case SDLK_c: {
                            mostrar_cuadrados = !mostrar_cuadrados;
                            dibujo(pantalla);
                        }
                        break;
                        case SDLK_r: {
                            dibujo(pantalla);
                        }
                        break;
                        /* //esta funcionalidad no me interesa aqu por ahora
                        case SDLK_x: {
                            lista.continuidad = !lista.continuidad;
                            if(lista.continuidad){
                                BEZIER_acomodarContinua(&lista);
                                BEZIER_recuperarNodos2d(&lista, x, y);
                            }
                            dibujo(pantalla);
                        }
                        break;*/
                    }
                    break;
                    
                case SDL_QUIT:
                    corriendo = 0;
                    break;
            }
        }
    }

    SDL_Quit();
    return 0;
}
