// /c09/trazadores/main.c
#include <SDL/SDL.h>
#include "tic_.h"
#include <SDL/SDL_gfxPrimitives.h>
#include <stdio.h>

#define ANCHO 640
#define ALTO  480

#define ANCHO_RECTANGULO 10

#define XMIN  0.0
#define XMAX 15.0
#define YMIN  0.0
#define YMAX  3.0
#define TPASO 0.005

#define TAM 21

static inline int x_real(double x_virtual){
    return (int) (ANCHO * x_virtual / XMAX);
}
static inline int y_real(double y_virtual){
    return (int) (ALTO * (1.0 - y_virtual / YMAX));
}
static inline double x_virtual(int x_real){
    return (XMAX * x_real) / ANCHO;
}
static inline double y_virtual(int y_real){
    return YMAX * (1.0 - y_real / (double)ALTO);
}

//Variables globales
double x[TAM] = {0.9, 1.3, 1.9, 2.1, 2.6, 3.0, 3.9, 4.4, 4.7, 5.0, 6.0, 7.0, 8.0, 9.2, 10.5, 11.3, 11.6, 12.0, 12.6, 13.0, 13.3};
double y[TAM] = {1.3, 1.5, 1.85, 2.1, 2.6, 2.7, 2.4, 2.15, 2.05, 2.1, 2.25, 2.3, 2.25, 1.95, 1.4, 0.9, 0.7, 0.6, 0.5, 0.4, 0.25};
trazadoresP *tr = NULL;
int i, j, k, l;
double tvar, xvar, yvar;
Uint32 color_fondo, color1, color2;

Uint32 gfxColor(Uint8 r, Uint8 g, Uint8 b){
    return 
        r << 24 |
        g << 16 |
        b << 8  |
        255;    //este valor es la opacidad del color
                //y debe ser mxima para que sea slido
}

void dibujo(SDL_Surface *pantalla){
    i=j=k=l=0;
    SDL_Rect rect;
    
    //Borra la pantalla
    SDL_FillRect(pantalla, NULL, color_fondo);
    
    //dibujo de los rectngulos
    rect.w = rect.h = ANCHO_RECTANGULO;
    for(i=0; i<=tr->num_trazadores; i++){
        rect.x = x_real(tr->x[i])-ANCHO_RECTANGULO/2;
        rect.y = y_real(tr->y[i])-ANCHO_RECTANGULO/2;
        rect.h = rect.w = ANCHO_RECTANGULO;
        rectangleColor(pantalla, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h, color2);
    }
    
    //dibujo de la curva
    tvar = 0.0;
    TIC_calcularP(tr, tvar, &xvar, &yvar);
    i=x_real(xvar);
    j=y_real(yvar);
    for(tvar = TPASO; tvar <= 1; tvar+=TPASO){
        if(!TIC_calcularP(tr, tvar, &xvar, &yvar)){
            aalineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color1);
            i=k; j=l;
        }
    }
    TIC_calcularP(tr, 1.0, &xvar, &yvar);
    aalineColor(pantalla, i, j, k=x_real(xvar), l=y_real(yvar), color1);
    
    //vuelca el buffer en la pantalla:
    SDL_Flip (pantalla);
}

int main(int argc, char *argv[]){
    SDL_Surface *pantalla = NULL;
    SDL_Event evento;
    int profundidad_color;
    const SDL_VideoInfo *info;
    int i;
    
    int corriendo = 1;
    int seleccionado = 0;
    
    if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
        fprintf(stderr, "No se puede iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);
    
    //Este if es importante para poder usar SDL_gfx
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
        //printf("%d\n", profundidad_color);
    } else {
        profundidad_color = 16;
    }
    
    pantalla = SDL_SetVideoMode(ANCHO, ALTO, profundidad_color, SDL_SWSURFACE);
    if(pantalla == NULL)
    {
        fprintf(stderr, "No se puede establecer el modo de video %dx%d: %s\n",
            ANCHO, ALTO, SDL_GetError());
        exit(1);
    }
    SDL_WM_SetCaption("Trazadores Interpolantes Cbicos Paramtricos!", NULL);
    
    color_fondo = SDL_MapRGB (pantalla->format,0,0,0);
    color1 = gfxColor(255,255,255);
    color2 = gfxColor(128,128,128);
    if((tr = TIC_crear_trazadoresP(x, y, TAM))==NULL){
        fprintf(stderr, "Error al crear los trazadores\n");
        exit(1);
    }
    TIC_imprimirP(tr, stderr);
    
    dibujo(pantalla);
    
    while(corriendo) {
        while(SDL_PollEvent(&evento)) {
            switch(evento.type){
                case SDL_MOUSEMOTION:{
                    if(seleccionado){
                        //actualizar el punto
                        tr->x[i]=x_virtual(evento.button.x);
                        tr->y[i]=y_virtual(evento.button.y);
                        TIC_actualizar_trazadorP(tr);
                        dibujo(pantalla);
                    }
                }
                    break;
                case SDL_MOUSEBUTTONDOWN:{
                    for(i=0; i<=tr->num_trazadores; i++){
                        if(evento.button.button == SDL_BUTTON_LEFT && // si hace clic sobre un nodo...
                            ((evento.button.x > x_real(tr->x[i])-ANCHO_RECTANGULO/2) &&
                             (evento.button.y > y_real(tr->y[i])-ANCHO_RECTANGULO/2) &&
                             (evento.button.x < x_real(tr->x[i])+ANCHO_RECTANGULO/2) &&
                             (evento.button.y < y_real(tr->y[i])+ANCHO_RECTANGULO/2))
                        ){
                            //se selecciona y el ndice queda en 'i'
                            seleccionado = 1;
                            //printf("seleccionado\n");
                            break;
                        }
                    }
                }
                    break;
                case SDL_MOUSEBUTTONUP:
                    seleccionado = 0;
                    break;
                
                case SDL_QUIT:
                    corriendo = 0;
                    break;
            }
        }
    }

    SDL_Quit();
    return 0;
}
