// /c09/trazadores/tic.c
#include "tic_.h"
#include <stdio.h>
#include <stdlib.h>

trazadores *TIC_crear_trazadores(double x[], double y[], int num_nodos){
    trazadores *tr = NULL;
    int i;
    
    if((tr = (trazadores*)malloc(sizeof(trazadores)))==NULL){
        fprintf(stderr, TIC_MSG_ERROR_MEMORIA);
        return NULL;
    }
    tr->x = tr->y = tr->a = tr->b = tr->c = tr->d = NULL;
    
    /*
    'x', 'y' y 'a' requieren 'num_nodos' elementos,
    'b', 'c' y 'd' slo 'num_nodos-1=num_trazadores',
    pero si 'c' tiene 'num_nodos', se simplifica la parte final del algoritmo
    */
    if((tr->x = (double*)malloc(5 * num_nodos * sizeof(double)))==NULL){
        fprintf(stderr, TIC_MSG_ERROR_MEMORIA);
        TIC_liberar_memoria(tr);
        free(tr);
        return NULL;
    }
    
    for(i=0; i < 5 * num_nodos; i++)
        tr->x[i]=0.0;
    
    tr->a = tr->y =
            tr->x + num_nodos;
    tr->b = tr->a + num_nodos;
    tr->c = tr->b + num_nodos;
    tr->d = tr->c + num_nodos;
    
    tr->num_trazadores = num_nodos - 1;
    
    if(!TIC_modificar_trazador(tr, x, y))
        return tr;
    else{
        TIC_liberar_memoria(tr);
        free(tr);
        return NULL;
    }
}

int TIC_modificar_trazador(trazadores* tr, double x[], double y[]){
    int i;
    int n;
    double *tmp[5];
    
    //Verificar integridad de los parmetros
    if(tr==NULL || 
        tr->a == NULL || tr->b == NULL || tr->c == NULL || tr->d == NULL || 
        x==NULL || y == NULL ||
        tr->num_trazadores < 1)
    {
        fprintf(stderr, TIC_MSG_ERROR_PARAMETROS);
        return TIC_COD_ERROR_PARAMETROS;
    }
    
    //Gestionar memoria temporal para el algoritmo
    if(
        (tmp[0] = (double*)malloc(5 * tr->num_trazadores * sizeof(double)))
        ==NULL)
    {
        fprintf(stderr, TIC_MSG_ERROR_MEMORIA);
        return TIC_COD_ERROR_MEMORIA;
    }
    for(i=1; i<5; i++)
        tmp[i] = tmp[0] + i * tr->num_trazadores;
    
    
    //Clculo de los trazadores
    n = tr->num_trazadores;
    for(i=0; i<=n; i++){
        tr->x[i] = x[i];
        tr->a[i] = y[i];
    }
    //paso 2...
    for(i=0; i<n; i++)
        tmp[H][i] = tr->x[i+1] - tr->x[i];
    //paso 3...
    for(i=1; i<n; i++)
        tmp[ALFA][i] = 3 * (
            (tr->a[i+1] - tr->a[i]) / tmp[H][i] -
            (tr->a[i] - tr->a[i-1]) / tmp[H][i-1]
        );
    //paso 4...
    tmp[L][0] = 1;
    tmp[MIU][0] = 0;
    tmp[Z][0] = 0;

    //paso 5...
    for(i=1; i<n; i++){
        tmp[L][i] = 2 * (tr->x[i+1] - tr->x[i-1]) - tmp[H][i-1]*tmp[MIU][i-1];
        tmp[MIU][i] = tmp[H][i] / tmp[L][i];
        tmp[Z][i] = (tmp[ALFA][i] - tmp[H][i-1]*tmp[Z][i-1]) / tmp[L][i];
    }
    //paso 6...
    //tmp[L][n] = 1;
    //tmp[Z][n] = 0;
    tr->c[n] = 0;
    
    //paso 7...
    for(i=n-1; i>=0; i--){
        tr->c[i] = tmp[Z][i] - tmp[MIU][i] * tr->c[i+1];
        tr->b[i] = (tr->a[i+1] - tr->a[i]) / tmp[H][i] 
            - tmp[H][i] * (tr->c[i+1] + 2*tr->c[i]) / 3;
        tr->d[i] = (tr->c[i+1] - tr->c[i]) / (3*tmp[H][i]);
    }
    
    //Liberar memoria temporal
    free(tmp[0]);
    
    return TIC_COD_EXITO;
}

int TIC_calcular(trazadores *tr, double x, double *y){
    
    if(tr==NULL || y == NULL){
        fprintf(stderr, TIC_MSG_ERROR_PARAMETROS);
        return TIC_COD_ERROR_PARAMETROS;
    }
    
    if(x >= tr->x[0]){
        /*
        int j=0;
        
        while(j < tr->num_trazadores && x > tr->x[j+1])
            j++;
        if(j < tr->num_trazadores){
            double tmpX = x - tr->x[j];
            *y = tr->a[j] + tmpX * ( tr->b[j] + tmpX * ( tr->c[j] + tmpX * tr->d[j] ) );
            return TIC_COD_EXITO;
        }
        */
        
        //Esta alternativa es ms eficiente que la de arriba:
        int j=1;
        
        while(j <= tr->num_trazadores && x > tr->x[j])
            j++;
        if(--j < tr->num_trazadores){
            double tmpX = x - tr->x[j];
            *y = tr->a[j] + tmpX * ( tr->b[j] + tmpX * ( tr->c[j] + tmpX * tr->d[j] ) );
            return TIC_COD_EXITO;
        }
    }
    
    //fprintf(stderr, TIC_MSG_ERROR_DOMINIO);
    return TIC_COD_ERROR_DOMINIO;
    
}

void TIC_liberar_memoria(trazadores *tr){
    free(tr->x);
}

void TIC_imprimir(trazadores *tr, FILE *f){
    int i;
    fprintf(f,"\nTrazadores:\n");
    fprintf(f,"i\tx_i\ty_i\ta_i\tb_i\tc_i\td_i\n");
    for(i=0; i<=tr->num_trazadores; i++)
        fprintf(f,"%d\t%3.2g\t%3.2g\t%3.2g\t%3.2g\t%3.2g\t%3.2g\n",
        i, tr->x[i], tr->y[i],
        tr->a[i], tr->b[i], tr->c[i], tr->d[i]);
}

/**** Paramtricos *********************/
trazadoresP *TIC_crear_trazadoresP(double x[], double y[], int num_nodos){
    trazadoresP *tr = NULL;
    int i;
    
    if((tr = (trazadoresP*)malloc(sizeof(trazadoresP)))==NULL){
        fprintf(stderr, TIC_MSG_ERROR_MEMORIA);
        return NULL;
    }
    tr->x = tr->y = tr->t 
        = tr->ax = tr->bx = tr->cx = tr->dx
        = tr->ay = tr->by = tr->cy = tr->dy = NULL;
    
    /*
    'x', 'y' y 'a' requieren 'num_nodos' elementos,
    'b', 'c' y 'd' slo 'num_nodos-1=num_trazadores',
    pero si 'c' tiene 'num_nodos', se simplifica la parte final del algoritmo,
    por lo que todos tendrn el mismo tamao de 'num_nodos'.
    */
    if((tr->t = (double*)malloc(9 * num_nodos * sizeof(double)))==NULL){
        fprintf(stderr, TIC_MSG_ERROR_MEMORIA);
        free(tr);
        return NULL;
    }
    
    for(i=0; i < 9 * num_nodos; i++)
        tr->t[i]=0.0;
    
    tr->ax = tr->x =
             tr->t + num_nodos;
    tr->bx = tr->ax + num_nodos;
    tr->cx = tr->bx + num_nodos;
    tr->dx = tr->cx + num_nodos;

    tr->ay = tr->y =
             tr->dx + num_nodos;
    tr->by = tr->ay + num_nodos;
    tr->cy = tr->by + num_nodos;
    tr->dy = tr->cy + num_nodos;
    
    tr->num_trazadores = num_nodos - 1;
    
    if(!TIC_modificar_trazadorP(tr, x, y))
        return tr;
    else{
        TIC_liberar_memoriaP(tr);
        free(tr);
        return NULL;
    }
}

int TIC_calcularP(trazadoresP *tr, double t, double *x, double *y){
    
    if(tr==NULL || x == NULL || y == NULL){
        fprintf(stderr, TIC_MSG_ERROR_PARAMETROS);
        return TIC_COD_ERROR_PARAMETROS;
    }
    
    if(t >= tr->t[0]){
        int j=1;
        
        while(j <= tr->num_trazadores && t > tr->t[j])
            j++;
        if(--j < tr->num_trazadores){
            double tmpT = t - tr->t[j];
            *x = tr->ax[j] + tmpT * ( tr->bx[j] + tmpT * ( tr->cx[j] + tmpT * tr->dx[j] ) );
            *y = tr->ay[j] + tmpT * ( tr->by[j] + tmpT * ( tr->cy[j] + tmpT * tr->dy[j] ) );
            return TIC_COD_EXITO;
        }
    }
    
    //fprintf(stderr, TIC_MSG_ERROR_DOMINIO);
    return TIC_COD_ERROR_DOMINIO;
    
}


int TIC_actualizar_trazadorP(trazadoresP* tr){
    return TIC_modificar_trazadorP(tr, tr->ax, tr->ay);
}

int TIC_modificar_trazadorP(trazadoresP* tr, double x[], double y[]){
    int i;
    int n;
    double *tmp[5];
    
    //Verificar integridad de los parmetros
    if(tr==NULL || 
        tr->ax == NULL || tr->bx == NULL || tr->cx == NULL || tr->dx == NULL || 
        tr->ay == NULL || tr->by == NULL || tr->cy == NULL || tr->dy == NULL || 
        x == NULL || y == NULL ||
        tr->num_trazadores < 1)
    {
        fprintf(stderr, TIC_MSG_ERROR_PARAMETROS);
        return TIC_COD_ERROR_PARAMETROS;
    }
    
    //Gestionar memoria temporal para el algoritmo
    if(
        (tmp[0] = (double*)malloc(5 * tr->num_trazadores * sizeof(double)))
        ==NULL)
    {
        fprintf(stderr, TIC_MSG_ERROR_MEMORIA);
        return TIC_COD_ERROR_MEMORIA;
    }
    for(i=1; i<5; i++)
        tmp[i] = tmp[0] + i * tr->num_trazadores;
    
    //Clculo de los trazadores
    n = tr->num_trazadores;
    
    //Distribuir uniformemente el parametro 't'
    //y asignar los coeficientes 'a'
    for(i=0; i<=n; i++){
        tr->t[i] = i / (float)n;
        tr->ax[i] = x[i];
        tr->ay[i] = y[i];
    }
    //paso 2...
    for(i=0; i<n; i++)
        tmp[H][i] = tr->t[i+1] - tr->t[i];
/**** en x *****************/
    //paso 3...
    for(i=1; i<n; i++)
        tmp[ALFA][i] = 3 * (
            (tr->ax[i+1] - tr->ax[i]) / tmp[H][i] -
            (tr->ax[i] - tr->ax[i-1]) / tmp[H][i-1]
        );
    //paso 4...
    tmp[L][0] = 1;
    tmp[MIU][0] = 0;
    tmp[Z][0] = 0;

    //paso 5...
    for(i=1; i<n; i++){
        tmp[L][i] = 2 * (tr->t[i+1] - tr->t[i-1]) - tmp[H][i-1]*tmp[MIU][i-1];
        tmp[MIU][i] = tmp[H][i] / tmp[L][i];
        tmp[Z][i] = (tmp[ALFA][i] - tmp[H][i-1]*tmp[Z][i-1]) / tmp[L][i];
    }
    //paso 6...
    //tmp[L][n] = 1;
    //tmp[Z][n] = 0;
    tr->cx[n] = 0;
    
    //paso 7...
    for(i=n-1; i>=0; i--){
        tr->cx[i] = tmp[Z][i] - tmp[MIU][i] * tr->cx[i+1];
        tr->bx[i] = (tr->ax[i+1] - tr->ax[i]) / tmp[H][i] 
            - tmp[H][i] * (tr->cx[i+1] + 2*tr->cx[i]) / 3;
        tr->dx[i] = (tr->cx[i+1] - tr->cx[i]) / (3*tmp[H][i]);
    }
/**** en y *****************/
    //paso 3...
    for(i=1; i<n; i++)
        tmp[ALFA][i] = 3 * (
            (tr->ay[i+1] - tr->ay[i]) / tmp[H][i] -
            (tr->ay[i] - tr->ay[i-1]) / tmp[H][i-1]
        );
    //paso 4...
    tmp[L][0] = 1;
    tmp[MIU][0] = 0;
    tmp[Z][0] = 0;

    //paso 5...
    for(i=1; i<n; i++){
        tmp[L][i] = 2 * (tr->t[i+1] - tr->t[i-1]) - tmp[H][i-1]*tmp[MIU][i-1];
        tmp[MIU][i] = tmp[H][i] / tmp[L][i];
        tmp[Z][i] = (tmp[ALFA][i] - tmp[H][i-1]*tmp[Z][i-1]) / tmp[L][i];
    }
    //paso 6...
    //tmp[L][n] = 1;
    //tmp[Z][n] = 0;
    tr->cy[n] = 0;
    
    //paso 7...
    for(i=n-1; i>=0; i--){
        tr->cy[i] = tmp[Z][i] - tmp[MIU][i] * tr->cy[i+1];
        tr->by[i] = (tr->ay[i+1] - tr->ay[i]) / tmp[H][i] 
            - tmp[H][i] * (tr->cy[i+1] + 2*tr->cy[i]) / 3;
        tr->dy[i] = (tr->cy[i+1] - tr->cy[i]) / (3*tmp[H][i]);
    }
/*********************/
    
    //Liberar memoria temporal
    free(tmp[0]);
    
    return TIC_COD_EXITO;
}


void TIC_liberar_memoriaP(trazadoresP *tr){
    free(tr->t);
}

void TIC_imprimirP(trazadoresP *tr, FILE *f){
    int i;
    fprintf(f,"\nTrazadores Paramtricos:\n");
    fprintf(f,"i t_i\tx_i\ty_i\tax_i\tbx_i\tcx_i\tdx_i\tay_i\tby_i\tcy_i\tdy_i\n");
    for(i=0; i<=tr->num_trazadores; i++)
        fprintf(f,"%d->\t%3.2g\t%3.2g\t%3.2g\n\t%3.2g\t%3.2g\t%3.2g\t%3.2g\n\t%3.2g\t%3.2g\t%3.2g\t%3.2g\n",
        i, tr->t[i], tr->x[i], tr->y[i],
        tr->ax[i], tr->bx[i], tr->cx[i], tr->dx[i],
        tr->ay[i], tr->by[i], tr->cy[i], tr->dy[i]);
}
