// /c09/trazadores/tic_.h
#include <stdio.h>

#define TIC_COD_EXITO 0
#define TIC_MSG_EXITO "xito\n"
#define TIC_COD_ERROR_PARAMETROS -1
#define TIC_MSG_ERROR_PARAMETROS "Error de parmetro(s)\n"
#define TIC_COD_ERROR_DOMINIO -2
#define TIC_MSG_ERROR_DOMINIO "Error de dominio\n"
#define TIC_COD_ERROR_MEMORIA -3
#define TIC_MSG_ERROR_MEMORIA "Error de solicitud de memoria dinmica\n"

#define H    0
#define ALFA 1
#define L    2
#define MIU  3
#define Z    4

/*
    Contiene toda la informacin par calcular
    los trazadores.
    
    El clculo de los trazadores es el siguiente:
        S(x) = S[j](x) =
            a[j] + b[j]*(x-x[j]) + c[j]*(x-x[j])^2 + d[j]*(x-x[j])^3 =
            a[j] + (x-x[j]) * ( b[j] + (x-x[j]) * ( c[j] + (x-x[j]) * d[j] ) )
        para x[j] <= x <= x[j+1]
        (ntese que los clculos se hacen con
        el nodo ms cercano a 'x' por la izquierda)
        
    Se asume que:
        x[0] < x[1] < ... < x[num_trazadores]
*/
typedef struct trazadores{
    
    //coordenadas de los nodos
    double *x;
    //las imgenes 'y' estn en el arreglo 'a'
    double *y;
    
    //coeficientes de los polinomios
    double *a;
    double *b;
    double *c;
    double *d;
    
    //nmero de polinomios
    int num_trazadores;
    
    //siempre debe cumplirse la siguiente relacin:
    //num_nodos = num_trazadores + 1
    
} trazadores;


/*
Entradas:
    ('x','y') define los nodos
    'num_nodos' indica el nmero de nodos
Salida:
    Se devuelve un puntero a la estructura creada o NULL si hubo error.
*/
trazadores *TIC_crear_trazadores(double x[], double y[], int num_nodos);

/*
Toma el valor 'x' y calcula su imagen
en funcin del trazador que le corresponda
y la deposita en 'y'
*/
int TIC_calcular(trazadores *tr, double x, double *y);

/*
Entradas:
    ('x','y') define los nodos
Salida:
    Se devuelve un codigo de resultado.

Se asume que los arreglos son de la misma longitud entre s
y con el valor 'num_nodos' usado para crear a 'tr'
*/
int TIC_modificar_trazador(trazadores* tr, double x[], double y[]);


/*
    Libera la memoria dinmicamente solicitada de la estructura
*/
void TIC_liberar_memoria(trazadores *tr);

/*
Imprime en la salida especificada los valores
almacenados en la estructura.
*/
void TIC_imprimir(trazadores *tr, FILE *f);

/*************** Paramtricos 2D *****************************/

typedef struct trazadoresP{
    
    //coordenadas de los nodos
    double *t;

    //las imgenes 'x' estn en el arreglo 'ax'
    double *x;
    //coeficientes de los polinomios en x
    double *ax;
    double *bx;
    double *cx;
    double *dx;
    
    //las imgenes 'y' estn en el arreglo 'ay'
    double *y;
    //coeficientes de los polinomios en y
    double *ay;
    double *by;
    double *cy;
    double *dy;
    
    //nmero de polinomios
    int num_trazadores;
    
    //la siguiente relacin siempre debe cumplirse:
    //num_nodos = num_trazadores + 1
    
} trazadoresP;

trazadoresP *TIC_crear_trazadoresP(double x[], double y[], int num_nodos);

/*
Toma el valor 't' y calcula su imagen
en funcin del trazador que le corresponda
y la deposita en ('x','y')
*/
int TIC_calcularP(trazadoresP *tr, double t, double *x, double *y);

/*
Se asume que los arreglos son de la misma longitud entre s
y con el valor 'num_nodos' usado para crear 'tr'
*/
int TIC_modificar_trazadorP(trazadoresP* tr, double x[], double y[]);

/*
Cuando se han actualizado los valores de los nodos
pero no se han agregado o quitado.
Se recalculan los coeficientes.
*/
int TIC_actualizar_trazadorP(trazadoresP* tr);

void TIC_liberar_memoriaP(trazadoresP *tr);

void TIC_imprimirP(trazadoresP *tr, FILE *f);
