// /c12/julia-mandelbrot/calculos.c
#include "calculos.h"

#define NUM_COLORES 50
Uint32 paleta[NUM_COLORES];

#include <stdlib.h>

static void generarPaletaAleatoria(void){
    int i;
    srand(getpid());
    for(i=0 ;i<NUM_COLORES; i++)
        paleta[i] = colorGfx(rand()%256,rand()%256,rand()%256);
}

static void generarPaletaRoja(void){
    int i;
    for(i=0 ;i<NUM_COLORES; i++)
        paleta[i] = colorGfx(255-i, 0,0);
}

void dibujarFractalJulia(SDL_Surface *pantalla, complejo *c, escala *e){
    int i, j;
    complejo z0, z;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        z0.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            z0.r = tV_Rx(e, j);
            
            z = z0;
            numPasos = 0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i)<LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else{
                pixelColor(pantalla, j, i, COLOR_AFUERA);
            }
        }
    }
    //~ lineColor(pantalla, 0, pantalla->h/2, pantalla->w, pantalla->h/2, VERDE);
    //~ lineColor(pantalla, pantalla->w/2, 0, pantalla->w/2, pantalla->h, VERDE);
    //~ lineColor(pantalla, tR_Vx(e,e->XminV), tR_Vy(e,e->YminV), tR_Vx(e,e->XmaxV), tR_Vy(e,e->YmaxV), AZUL);
    //~ lineColor(pantalla, tR_Vx(e,e->XminV), tR_Vy(e,e->YmaxV), tR_Vx(e,e->XmaxV), tR_Vy(e,e->YminV), AZUL);
}

void dibujarFractalJulia2(SDL_Surface *pantalla, complejo *c, escala *e){
    int i, j;
    complejo z0, z;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        z0.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            z0.r = tV_Rx(e, j);
            
            z = z0;
            numPasos = 0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i)<LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else{
                pixelColor(pantalla, j, i, colorGfx((MAXPASOS-numPasos)*256/MAXPASOS,0,0));
            }
        }
    }
}

void dibujarFractalJulia3(SDL_Surface *pantalla, complejo *c, escala *e){
    int i, j;
    complejo z0, z;
    int numPasos;
    
    generarPaletaAleatoria();
    
    for(i=0; i<e->anchoReal; i++){
        z0.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            z0.r = tV_Rx(e, j);
            
            z = z0;
            numPasos = 0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i)<LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else{
                pixelColor(pantalla, j, i, (numPasos<NUM_COLORES)?paleta[numPasos]:BLANCO);
            }
        }
    }
}

void dibujarFractalJulia4(SDL_Surface *pantalla, complejo *c, escala *e){
    int i, j;
    complejo z0, z;
    int numPasos;
    
    generarPaletaRoja();
    
    for(i=0; i<e->anchoReal; i++){
        z0.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            z0.r = tV_Rx(e, j);
            
            z = z0;
            numPasos = 0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i)<LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, (z.r>0.0)?((z.i>0.0)?ROJO:AZUL):((z.i>0.0)?VERDE:ROJO|VERDE));
            }
            else{
                pixelColor(pantalla, j, i, (numPasos<NUM_COLORES)?paleta[numPasos]:BLANCO);
            }
        }
    }
}

void dibujarFractalJulia5(SDL_Surface *pantalla, complejo *c, escala *e){
    int i, j;
    complejo z0, z;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        z0.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            z0.r = tV_Rx(e, j);
            
            z = z0;
            numPasos = 0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i)<LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else{
                pixelColor(pantalla, j, i, colorGfx(0,(MAXPASOS-numPasos/2)*256/MAXPASOS,0));
            }
        }
    }
}

void dibujarFractalJulia6(SDL_Surface *pantalla, complejo *c, escala *e){
    int i, j;
    complejo z0, z;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        z0.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            z0.r = tV_Rx(e, j);
            
            z = z0;
            numPasos = 0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i)<LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else{
                pixelColor(pantalla, j, i, colorGfx(
                    (MAXPASOS-numPasos)*256/MAXPASOS,
                    (MAXPASOS-numPasos)*256/MAXPASOS,
                    (MAXPASOS-numPasos)*256/MAXPASOS));
            }
        }
    }
}

void dibujarFractalMandelbrot(SDL_Surface *pantalla, escala *e){
    //lineColor(pantalla, tR_Vx(e,0), tR_Vy(e,0), tR_Vx(e,c->r), tR_Vy(e,c->i), VERDE);
    //lineColor(pantalla, tR_Vx(e,e->XminV), tR_Vy(e,e->YminV), tR_Vx(e,e->XmaxV), tR_Vy(e,e->YmaxV), VERDE);
    
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else{
                pixelColor(pantalla, j, i, COLOR_AFUERA);
            }
        }
    }
}

void dibujarFractalMandelbrot2(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, NEGRO);
            }
            else
                if(numPasos%2){
                    pixelColor(pantalla, j, i, BLANCO);
                }
                else{
                    pixelColor(pantalla, j, i, NEGRO);
                }
        }
    }
}

void dibujarFractalMandelbrot3(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else
                if(z.i<0.0){
                    pixelColor(pantalla, j, i, BLANCO);
                }
                else{
                    pixelColor(pantalla, j, i, NEGRO);
                }
        }
    }
}

void dibujarFractalMandelbrot4(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else
                pixelColor(pantalla, j, i, colorGfx(numPasos*256/MAXPASOS,0,0));
        }
    }
}

void dibujarFractalMandelbrot5(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else
                pixelColor(pantalla, j, i, colorGfx(
                    numPasos*256/MAXPASOS,
                    (MAXPASOS-numPasos)*256/MAXPASOS,
                    0));
        }
    }
}

void dibujarFractalMandelbrot6(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else
                pixelColor(pantalla, j, i, colorGfx(
                    numPasos*256/MAXPASOS,
                    numPasos*256/MAXPASOS,
                    numPasos*256/MAXPASOS));
        }
    }
}

void dibujarFractalMandelbrot7(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else
                pixelColor(pantalla, j, i, colorGfx(
                    numPasos*256/MAXPASOS,
                    (MAXPASOS-numPasos)*256/MAXPASOS,
                    numPasos*256/MAXPASOS));
        }
    }
}

void dibujarFractalMandelbrot8(SDL_Surface *pantalla, escala *e){
    int i, j;
    complejo z={0.0, 0.0}, c;
    int numPasos;
    
    for(i=0; i<e->anchoReal; i++){
        c.i = tV_Ry(e, i);
        
        for(j=0; j<e->altoReal; j++){
            c.r = tV_Rx(e, j);
            
            numPasos = 0;
            z.r = z.i = 0.0;
            while(numPasos<MAXPASOS && (sq(z.r)+sq(z.i) < LIMITEMODULO2)){
                CX_suma(CX_cuadrado(&z), &c);
                numPasos++;
            }
            if(numPasos==MAXPASOS){
                pixelColor(pantalla, j, i, COLOR_ADENTRO);
            }
            else
                pixelColor(pantalla, j, i, colorGfx(
                    (MAXPASOS-numPasos)*256/MAXPASOS,
                    (MAXPASOS-numPasos)*256/MAXPASOS,
                    (MAXPASOS-numPasos)*256/MAXPASOS));
        }
    }
}
