// /c12/julia-mandelbrot/mandelbrot.c
#include "calculos.h"
#include "valoresIniciales.h"
#include <stdio.h>
#include <stdlib.h>

#define ANCHO 700
#define ALTO 700

#define XMINV -2.5
#define XMAXV 1.5
#define YMINV -2.0
#define YMAXV 2.0

SDL_Surface *pantalla = NULL;
escala e;
const Uint32 colorFondo = BLANCO;
void (*dibujar_mandelbrot[NUM_FUNCIONES_MANDELBROT])(SDL_Surface *, escala *) = {
    dibujarFractalMandelbrot,
    dibujarFractalMandelbrot2,
    dibujarFractalMandelbrot3,
    dibujarFractalMandelbrot4,
    dibujarFractalMandelbrot5,
    dibujarFractalMandelbrot6,
    dibujarFractalMandelbrot7,
    dibujarFractalMandelbrot8};
void (*dibujarMandelbrot)(SDL_Surface *, escala *);

void limpieza(void){
    SDL_Quit();
}

void dibujarContenido(void){
    borrarPantalla(pantalla, colorFondo);
    stringColor(pantalla, 0,0, "Por favor espere...", colorGfx(255,128,128));
    SDL_Flip(pantalla);
    
    dibujarMandelbrot(pantalla, &e);
    //------
    //system("clear");
    //printf("listo\n");
    SDL_Flip(pantalla);
}

void iniciarModoVideo(void){
    const SDL_VideoInfo *info;
    int profundidad_color;
    
    if (SDL_Init(SDL_INIT_VIDEO) < 0){
        printf("Error al iniciar SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(limpieza);
    
    //Este if es importante para poder usar SDL_gfx
    info = SDL_GetVideoInfo();
    if ( info->vfmt->BitsPerPixel > 8 ) {
        profundidad_color = info->vfmt->BitsPerPixel;
        //printf("Profundidad del color:%d\n", profundidad_color);
    } else {
        profundidad_color = 16;
    }
    pantalla = SDL_SetVideoMode(ANCHO, ALTO, profundidad_color,  SDL_SWSURFACE);
    if (pantalla == NULL){
        printf("Error al inicializar el modo de video: '%s'\n", SDL_GetError());
        exit(2);
    }
}

void configuracionesIniciales(void){
    //...inicializacin de variables, colores, contadores, etc.
    e.anchoReal = ANCHO;
    e.altoReal = ALTO;
    e.XminV = XMINV;
    e.XmaxV = XMAXV;
    e.YminV = YMINV;
    e.YmaxV = YMAXV;
    corregirDireccion(&e);
}

int main(int argc, char *argv[]){
    SDL_Event evento;
    int corriendo = 1;
    int punto = 0;
    int x0, y0, x1, y1;
    float x0V, y0V, x1V, y1V;
    int ix;
    char titulo[MAXLONG];
    
    iniciarModoVideo();
    configuracionesIniciales();
    
    SDL_WM_SetCaption("Conjunto de Mandelbrot", NULL);
    
    //inhabilita el encolamiento de nuevos eventos
    SDL_EventState((Uint8) SDL_ALLEVENTS, SDL_DISABLE);
    
    //hacer primer dibujo
    dibujarMandelbrot = dibujar_mandelbrot[ix=0];
    sprintf(titulo, TXT_MT2, TXT_J1, ix+1, NUM_FUNCIONES_MANDELBROT);
    SDL_WM_SetCaption(titulo, NULL);
    dibujarContenido();
    
    //rehabilita el encolamiento de nuevos eventos
    SDL_EventState((Uint8) SDL_ALLEVENTS, SDL_ENABLE);
    
    while(corriendo) {
        while(SDL_PollEvent(&evento)) {
            switch(evento.type){
                case SDL_MOUSEBUTTONDOWN:{
                    //inhabilita el encolamiento de nuevos eventos
                    SDL_EventState((Uint8) SDL_ALLEVENTS, SDL_DISABLE);
                    
                    if(evento.button.button==SDL_BUTTON_RIGHT){
                        e.XminV = XMINV;
                        e.YminV = YMINV;
                        e.XmaxV = XMAXV;
                        e.YmaxV = YMAXV;
                        corregirDireccion(&e);
                        
                        dibujarMandelbrot = dibujar_mandelbrot[ix = (++ix%NUM_FUNCIONES_MANDELBROT)];
                        sprintf(titulo, TXT_MT2, TXT_J1, ix+1, NUM_FUNCIONES_MANDELBROT);
                        SDL_WM_SetCaption(titulo, NULL);
                        dibujarContenido();
                    }
                    else if(evento.button.button==SDL_BUTTON_LEFT){
                        if(punto == 0){
                            x0 = evento.button.x;
                            y0 = evento.button.y;
                            punto++;
                        }
                        else if(punto==1){
                            x1 = evento.button.x;
                            y1 = evento.button.y;
                            boxColor(pantalla, x0, y0, x1, y1,COLOR_CUADRO);
                            SDL_Flip(pantalla);
                            SDL_Delay(1000);
                            
                            x0V = tV_Rx(&e, x0);
                            y0V = tV_Ry(&e, y0);
                            x1V = tV_Rx(&e, x1);
                            y1V = tV_Ry(&e, y1);
                            e.XminV = x0V;
                            e.YminV = y0V;
                            e.XmaxV = x1V;
                            e.YmaxV = y1V;
                            corregirDireccion(&e);
                            
                            punto=0;
                            x0=y0=x1=y1=0;
                            dibujarContenido();
                        }
                    }
                    else if(evento.button.button==SDL_BUTTON_MIDDLE){
                        e.XminV = XMINV;
                        e.XmaxV = XMAXV;
                        e.YminV = YMINV;
                        e.YmaxV = YMAXV;
                        corregirDireccion(&e);
                        
                        dibujarContenido();
                    }
                    
                    //rehabilita el encolamiento de nuevos eventos
                    SDL_EventState((Uint8) SDL_ALLEVENTS, SDL_ENABLE);
                }
                break;
                
                case SDL_QUIT:
                    corriendo = 0;
                break;
            }
        }
    }
    return 0;
}
