package proyectoGPC;
/**
* Almacena una matriz de transformacin
*
*/
import java.awt.*;
public class Matriz3d implements Config3D{ // Definir una clase de Matrices 3-D

	//contenido
    float e[][] = new float[4][4];

    static final double aRad = Math.PI/180.0;
    static int REEMPLAZAR = 0;
    static int PRECONCAT = 1;
    static int POSTCONCAT = 2;

//	configuracin
	int modo_proyeccion = MODO_PROYECCION;
    Punto3d vision = new Punto3d(0f, 0f, 0f);
	float distancia = 0f;
//	necesitamos una transformacin inversa para el punto de iluminacin
// y por defecto no est�inicializada
	Matriz3d inv; boolean tiene_inversa = false;

    public Matriz3d(){           // Crear una matriz identidad
        identidad ();
    }

    public void identidad(){      // Hacer unan identidad matrix
        e[0][1] = e[0][2] = e[0][3] = 0.0f;
        e[1][0] = e[1][2] = e[1][3] = 0.0f;
        e[2][0] = e[2][1] = e[2][3] = 0.0f;
        e[3][0] = e[3][2] = e[3][2] = 0.0f;
        e[0][0] = 1.0f;
        e[1][1] = 1.0f;
        e[2][2] = 1.0f;
    }

    public Point transformar2D(Punto3d p){
        float x = p.x * e[0][0] + p.y * e[0][1] + p.z * e[0][2] + e[0][3];
        float y = p.x * e[1][0] + p.y * e[1][1] + p.z * e[1][2] + e[1][3];
        float z = p.x * e[2][0] + p.y * e[2][1] + p.z * e[2][2] + e[2][3];
		if((modo_proyeccion & PERSPECTIVE)==PERSPECTIVE){
    	    x = distancia*x/z;
    	    y = distancia*y/z;
		}
        return(new Point((int)x,(int)y));
    }

	//	Transformar un v�tice mediante una matriz
	public void transformar(Vertice vert){
        vert.xp = vert.x * e[0][0] + vert.y * e[0][1] + vert.z * e[0][2] + e[0][3];
        vert.yp = vert.x * e[1][0] + vert.y * e[1][1] + vert.z * e[1][2] + e[1][3];
        vert.zp = vert.x * e[2][0] + vert.y * e[2][1] + vert.z * e[2][2] + e[2][3];

		if((modo_proyeccion & PERSPECTIVE)==PERSPECTIVE){
    	    vert.xp = distancia*vert.xp/vert.zp;
    	    vert.yp = distancia*vert.yp/vert.zp;
		}
    }

 	//	Transformar un punto mediante una matriz
	public Punto3d transformar(Punto3d p){
     	Punto3d punto = new Punto3d(0f,0f,0f);
     	punto.x = p.x * e[0][0] + p.y * e[0][1] + p.z * e[0][2] + e[0][3];
        punto.y = p.x * e[1][0] + p.y * e[1][1] + p.z * e[1][2] + e[1][3];
        punto.z = p.x * e[2][0] + p.y * e[2][1] + p.z * e[2][2] + e[2][3];

		if((modo_proyeccion & PERSPECTIVE)==PERSPECTIVE){
    	    punto.x = distancia*punto.x/punto.z;
    	    punto.y = distancia*punto.y/punto.z;
		}
		return punto;
    }

    // Trasladar la matriz
    public void trasladar(float x, float y, float z){
        e[0][3] += x;
        e[1][3] += y;
        e[2][3] += z;
    }
    // Escalar una matriz por x,y,z
    public void escalar(float x, float y, float z){
        e[0][0] *= x;
        e[1][1] *= y;
        e[2][2] *= z;
    }

    // Escalar una matriz por un slo valor
    public void escalar(float s){
        e[0][0] *= s;
        e[1][1] *= s;
        e[2][2] *= s;
    }
    // Rotar una matriz theta grados sobre el eje x
    public void rotarX(double theta){
        theta *= aRad;    // Convertir el �gulo a radianes

        double theta_cos = Math.cos(theta);
        double theta_sin = Math.sin(theta);

        Matriz3d m = new Matriz3d();
        m.e[1][1] = (float)theta_cos;
        m.e[1][2] = (float)-theta_sin;
        m.e[2][1] = (float)theta_sin;
        m.e[2][2] = (float)theta_cos;
        this.multiplicar(m, Matriz3d.POSTCONCAT);
    }
    // Rotar la matriz theta grados sobre el eje y
    public void rotarY(double theta){
        theta *= aRad;                     // Convertir a  radianes
        double theta_cos = Math.cos(theta);
        double theta_sin = Math.sin(theta);

        Matriz3d m = new Matriz3d();
        m.e[0][0] = (float)theta_cos;
        m.e[0][2] = (float)theta_sin;
        m.e[2][0] = (float)-theta_sin;
        m.e[2][2] = (float)theta_cos;
        this.multiplicar(m, Matriz3d.POSTCONCAT);
    }
    // rotar la matriz theta grados sobre el eje Z
    void rotarZ(double theta){
        theta *= aRad;                       // Convertir a radianes
        double theta_cos = Math.cos(theta);
        double theta_sin = Math.sin(theta);

        Matriz3d m = new Matriz3d();
        m.e[0][0] = (float)theta_cos;
        m.e[0][1] = (float)-theta_sin;
        m.e[1][0] = (float)theta_sin;
        m.e[1][1] = (float)theta_cos;
        this.multiplicar(m, Matriz3d.POSTCONCAT);
    }
    // Concatenar (multiplicar) esta matriz con otra
    void multiplicar(Matriz3d matriz, int concat){
        Matriz3d A = null;
        Matriz3d B = null;
        if (concat == Matriz3d.PRECONCAT){
            A = matriz;
            B = this;
        } else if (concat == Matriz3d.POSTCONCAT){
            A = this;
            B = matriz;
        }
        else if (concat == Matriz3d.REEMPLAZAR){
            A = this;
            A.identidad();
            B = matriz;
        }

        float n00 = A.e[0][0]*B.e[0][0] + A.e[1][0]*B.e[0][1] + A.e[2][0]*B.e[0][2];
        float n01 = A.e[0][1]*B.e[0][0] + A.e[1][1]*B.e[0][1] + A.e[2][1]*B.e[0][2];
        float n02 = A.e[0][2]*B.e[0][0] + A.e[1][2]*B.e[0][1] + A.e[2][2]*B.e[0][2];
        float n03 = A.e[0][3]*B.e[0][0] + A.e[1][3]*B.e[0][1] + A.e[2][3]*B.e[0][2] + B.e[0][3];

        float n10 = A.e[0][0]*B.e[1][0] + A.e[1][0]*B.e[1][1] + A.e[2][0]*B.e[1][2];
        float n11 = A.e[0][1]*B.e[1][0] + A.e[1][1]*B.e[1][1] + A.e[2][1]*B.e[1][2];
        float n12 = A.e[0][2]*B.e[1][0] + A.e[1][2]*B.e[1][1] + A.e[2][2]*B.e[1][2];
        float n13 = A.e[0][3]*B.e[1][0] + A.e[1][3]*B.e[1][1] + A.e[2][3]*B.e[1][2] + B.e[1][3];

        float n20 = A.e[0][0]*B.e[2][0] + A.e[1][0]*B.e[2][1] + A.e[2][0]*B.e[2][2];
        float n21 = A.e[0][1]*B.e[2][0] + A.e[1][1]*B.e[2][1] + A.e[2][1]*B.e[2][2];
        float n22 = A.e[0][2]*B.e[2][0] + A.e[1][2]*B.e[2][1] + A.e[2][2]*B.e[2][2];
        float n23 = A.e[0][3]*B.e[2][0] + A.e[1][3]*B.e[2][1] + A.e[2][3]*B.e[2][2] + B.e[2][3];

        e[0][0] = n00;
        e[0][1] = n01;
        e[0][2] = n02;
        e[0][3] = n03;

        e[1][0] = n10;
        e[1][1] = n11;
        e[1][2] = n12;
        e[1][3] = n13;

        e[2][0] = n20;
        e[2][1] = n21;
        e[2][2] = n22;
        e[2][3] = n23;
    }
    //	crear una copia de esta matriz
    public Matriz3d copiar(){
		Matriz3d m = new Matriz3d();
        m.e[0][0] = e[0][0];
        m.e[0][1] = e[0][1];
        m.e[0][2] = e[0][2];
        m.e[0][3] = e[0][3];

        m.e[1][0] = e[1][0];
        m.e[1][1] = e[1][1];
        m.e[1][2] = e[1][2];
        m.e[1][3] = e[1][3];

        m.e[2][0] = e[2][0];
        m.e[2][1] = e[2][1];
        m.e[2][2] = e[2][2];
        m.e[2][3] = e[2][3];

		return m;
	}

/*
* Recibe el punto de visin en coordenadas esfericas (d, theta, phi)
* y un cuarto par�etro de rotacin del plano de visin
* con respecto al vector de punto de visin
*/
    public void colocarPuntodeVision(float theta, float phi, float omega, float d){
		double th = theta * aRad;
		double ph = phi * aRad;
		double th_cos = Math.cos(th);
		double th_sin = Math.sin(th);
		double ph_cos = Math.cos(ph);
		double ph_sin = Math.sin(ph);
		double d_xy = d * ph_sin;
		vision.x = (float) (d_xy * th_cos);
		vision.y = (float) (d_xy * th_sin);
		vision.z = (float) (d * ph_cos);
		distancia = d;

        this.identidad();
		this.trasladar(-vision.x, -vision.y, -vision.z);
        this.rotarX(90);
        this.rotarY(90 + theta);
        this.rotarX(90 - phi);
        this.rotarZ(omega);

    }
/*
* Recibe el punto de visin en coordenadas esfericas (d, theta, phi)
* donde la distancia est�en 'distancia' (variable miembro)
* y un cuarto par�etro de rotacin del plano de visin
* con respecto al vector de punto de visin
*/
    public void colocarPuntodeVision(float theta, float phi, float omega){
		double th = theta * aRad;
		double ph = phi * aRad;
		double th_cos = Math.cos(th);
		double th_sin = Math.sin(th);
		double ph_cos = Math.cos(ph);
		double ph_sin = Math.sin(ph);
		double d_xy = distancia * ph_sin;
		vision.x = (float) (d_xy * th_cos);
		vision.y = (float) (d_xy * th_sin);
		vision.z = (float) (distancia * ph_cos);

        this.identidad();
		this.trasladar(-vision.x, -vision.y, -vision.z);
        this.rotarX(90);
        this.rotarY(90 + theta);
        this.rotarX(90 - phi);
        this.rotarZ(omega);
    }


    void calcularInversa(){
	    float m[][] = new float[3][6], coef;
	    int i, j, k;

	    //copiando a registro temporal
	    for(i=0; i<3; i++){
		    for(j=0; j<3; j++)
			    m[i][j] = e[i][j];
		    for(j=3; j<6; j++)
			    m[i][j] = (i==(j-3))?1:0;
	    }

	    // pivoteando
	    for(i=0; i<3; i++){
		    if((coef = m[i][i]) == 0f)
			    if(i<2){//si no es la ltima fila
				    /*intercambiar con la fila siguiente,
				     * que tenga pivote distinto de cero*/
				    float aux[] = new float[6]; int h;
				    if(m[i+1][i] != 0.0f)
					    for(h=i; h<6; h++){
						    aux[h] = m[i][h];
						    m[i][h] = m[i+1][h];
						    m[i+1][h] = aux[h];
					    }
				    else if(i<1 && m[i+2][i] != 0f)
					    for(h=i; h<6; h++){
						    aux[h] = m[i][h];
						    m[i][h] = m[i+2][h];
						    m[i+2][h] = aux[h];
					    }
				    else{
					    //no tiene inversa
					    tiene_inversa = false;
					    return;
				    }
				    coef = m[i][i];
			    } else{//si es la ltima fila, no hay m� posibles pivotes
				    //por lo tanto, no tiene inversa
				    tiene_inversa = false;
				    return;
			    }
		    // fin de verificacin de pivote v�ido

		    for(j=i; j<6; j++)
			    //hacer 1 el pivote
			    m[i][j] /= coef;

		    for(k=i+1; k<(i+3); k++)
			    if((coef = m[k%3][i]) != 0)
				    for(j=i; j<6; j++)//Hacer '-coef*F(k%3) + F(i) -> F(i)'
					    m[k%3][j] -= coef * m[i][j];
	    }//fin de pivoteo

	    //sacando del registro temporal
	    inv = new Matriz3d();
	    for(i=0; i<3; i++)
		    for(j=0; j<3; j++)
			    inv.e[i][j] = m[i][j+3];
	    //la inversa existe
	    tiene_inversa = true;
	}
}	//fin de clase Matriz3d
