package proyectoGPC;
import java.awt.*;
import java.awt.image.*;
/**
 * Estructuras para el almacenamiento en memoria
 * representa un objeto tridimensional centrado en el origen
 *
 * @author (Eduardo Navas)
 * @version (0.1)
 */
public class ObjetoEnmallado{

	listaPoligonos poligonos = null;
    listaAristas aristas = null;
    listaVertices vertices = null;

//	la magnitud del punto ms distante al origen que pertenece a este objeto
	float magMasDistante = 0f;

//	constructores con acceso restringido
    protected ObjetoEnmallado(listaPoligonos poligonos, listaAristas aristas, listaVertices vertices, float magMasDistante){
        this.poligonos = poligonos;
        this.aristas = aristas;
        this.vertices = vertices;
        this.magMasDistante = magMasDistante;
    }
    protected ObjetoEnmallado(listaAristas[] poli, Arista[] aris, Vertice[] vert, float magMasDistante){
        this.poligonos = new listaPoligonos(poli);
        this.aristas = new listaAristas(aris);
        this.vertices = new listaVertices(vert);
        this.magMasDistante = magMasDistante;
    }
    public void transformar(Matriz3d m){
//	se propaga la transformacin a todos los vrtices del objeto
        this.vertices.transformar(m);
    }
}

class listaPoligonos{
    int ix = -1, numPoligonos = 0;
    nodoPoligono primero=null, pi=null;

    public listaPoligonos(listaAristas[] listas){
        nodoPoligono p;
        this.primero = new nodoPoligono(listas[0]);
        p = this.primero;
        numPoligonos = listas.length;

        for(int i=1; i<numPoligonos; i++){
            p.siguiente = new nodoPoligono(listas[i]);
            p.siguiente.anterior = p;
            p = p.siguiente;
        }
        p.siguiente = this.primero;
        this.primero.anterior = p;
        pi = this.primero;
        ix = 0;
    }
    public listaPoligonos(){
        numPoligonos = 0;
        primero = null;
    }
    public void agregar(listaAristas la){
        nodoPoligono nodo = new nodoPoligono(la);
        if(numPoligonos>0){
            primero.anterior.siguiente = nodo;
            nodo.anterior = primero.anterior;
            nodo.siguiente = primero;
            primero.anterior = nodo;
        }else{
            primero = nodo;
            primero.anterior = primero.siguiente = nodo;
        }
        numPoligonos++;
    }
    public void agregar(Arista[] a, boolean[] dirs){
        this.agregar(new listaAristas(a, dirs));
    }
/*	utilizamos la ordenacin rpida para poner los
	polgonos de ms lejano a ms cercano, para
	que los polgonos ms cercanos oculten a los
	ms lejanos.
*/
	public void ordenarPoligonos(){
		nodoPoligono nodos[] = new nodoPoligono[numPoligonos], p=primero;
		float[] dist = new float[numPoligonos];
		for(int i=0; i<numPoligonos; i++, p = p.siguiente){
			nodos[i] = p;
			dist[i] = p.aristas.promedioZ();
		}
		qsortPoligonos(nodos, dist, 0, numPoligonos-1);

//	 volver a configurar la lista
        p = this.primero = nodos[0];

        for(int i=1; i<numPoligonos; i++){
            p.siguiente = nodos[i];
            p.siguiente.anterior = p;
            p = p.siguiente;
        }
        p.siguiente = this.primero;
        this.primero.anterior = p;

		pi = primero;
		ix = 0;
	}
	void qsortPoligonos(nodoPoligono[] nodos, float[] dist, int inicio, int fin){
		int bajo = inicio,
			alto = fin;
		if(inicio >= fin)
			return;
		float zvm = dist[(inicio+fin)/2];	  //vrtice medio
		do{
			while(dist[bajo] > zvm)
				bajo++;
			while(dist[alto] < zvm)
				alto--;
			if(bajo<=alto){
				float ftmp = dist[bajo];
				dist[bajo] = dist[alto];
				dist[alto] = ftmp;

				nodoPoligono tmp = nodos[bajo];
				nodos[bajo] = nodos[alto];
				nodos[alto] = tmp;

				bajo++; alto--;
			}
		}while(bajo<=alto);
		qsortPoligonos(nodos, dist, inicio, alto);
		qsortPoligonos(nodos, dist, bajo, fin);
	}
}
class nodoPoligono{
	nodoPoligono anterior=null, siguiente=null;
/* Se asume que esta lista de aristas es tal,
* que el vrtice cabeza de una arista
* es el cola del siguiente, y adems,
* que la primera arista conecta con la ltima.
*/
	listaAristas aristas = null;
    float A=0, B=0, C=0, D=0;
    public nodoPoligono(listaAristas aristas){
        float[] coef = aristas.generarCoeficientesNormal();
        this.aristas = aristas;
        A=coef[0];
        B=coef[1];
		C=coef[2];
		D=coef[3];
	}
	public Punto3d Normal(){
	   return new Punto3d(A,B,C);
	}
}
class listaAristas{
    int ix=-1, numAristas=0;
    nodoArista primera=null, ai =null;
    public listaAristas(Arista[] aristas){
        nodoArista a;
        this.primera = new nodoArista(aristas[0]);
        a = this.primera;
        numAristas = aristas.length;

        for(int i=1; i<numAristas; i++){
            a.siguiente = new nodoArista(aristas[i]);
            a.siguiente.anterior = a;
            a = a.siguiente;
        }
        a.siguiente = this.primera;
        this.primera.anterior = a;
        ai = this.primera;
        ix = 0;
    }
    public listaAristas(Arista[] aristas, boolean[] dirs){
        nodoArista a;
        this.primera = new nodoArista(aristas[0], dirs[0]);
        a = this.primera;
        numAristas = aristas.length;

        for(int i=1; i<numAristas; i++){
            a.siguiente = new nodoArista(aristas[i], dirs[i]);
            a.siguiente.anterior = a;
            a = a.siguiente;
        }
        a.siguiente = this.primera;
        this.primera.anterior = a;
        ai = this.primera;
        ix = 0;
    }
    public listaAristas(){
        numAristas = 0;
        primera = null;
    }
    public void agregar(Arista a){
        nodoArista nodo = new nodoArista(a);
        if(numAristas>0){
            primera.anterior.siguiente = nodo;
            nodo.anterior = primera.anterior;
            nodo.siguiente = primera;
            primera.anterior = nodo;
        }else{
            primera = nodo;
            primera.anterior = primera.siguiente = nodo;
        }
        numAristas++;
    }
    public void agregar(Arista a, boolean dir){
        nodoArista nodo = new nodoArista(a, dir);
        if(numAristas>0){
            primera.anterior.siguiente = nodo;
            nodo.anterior = primera.anterior;
            nodo.siguiente = primera;
            primera.anterior = nodo;
        }else{
            primera = nodo;
            primera.anterior = primera.siguiente = nodo;
        }
        numAristas++;
    }
    public void agregar(Vertice v1, Vertice v2, int indice){
        this.agregar(new Arista(v1, v2, indice));
    }
    public void agregar(Vertice v1, Vertice v2, int indice, boolean dir){
        this.agregar(new Arista(v1, v2, indice), dir);
    }
    public void agregar(Arista[] aa){
        int ind;
        nodoArista[] na = new nodoArista[aa.length];
        for(ind = 0; ind < aa.length; ind++)
            na[ind] = new nodoArista(aa[ind]);
        for(ind = 1; ind < aa.length-1; ind++){
            na[ind].anterior = na[ind-1];
            na[ind].siguiente = na[ind+1];
        }
        if(numAristas>0){
            primera.anterior.siguiente = na[0];
            na[0].anterior = primera.anterior;
            na[aa.length-1].siguiente = primera;
            primera.anterior = na[aa.length-1];
        }else{
            this.primera = na[0];
            primera.anterior = na[aa.length-1];
            na[aa.length-1].siguiente = primera;
        }
        numAristas += aa.length;
    }
    public void agregar(Arista[] aa, boolean[] dirs){
        int ind;
        nodoArista[] na = new nodoArista[aa.length];
        for(ind = 0; ind < aa.length; ind++)
            na[ind] = new nodoArista(aa[ind], dirs[ind]);
        for(ind = 1; ind < aa.length-1; ind++){
            na[ind].anterior = na[ind-1];
            na[ind].siguiente = na[ind+1];
        }
        if(numAristas>0){
            primera.anterior.siguiente = na[0];
            na[0].anterior = primera.anterior;
            na[aa.length-1].siguiente = primera;
            primera.anterior = na[aa.length-1];
        }else{
            this.primera = na[0];
            primera.anterior = na[aa.length-1];
            na[aa.length-1].siguiente = primera;
        }
        numAristas += aa.length;
    }
/*	en algunos modos de presentacin, se dibujan
	las aristas de cada polgono, por lo que se corre
	el riesgo de dibujar las aristas comunes a dos
	o ms polgonos varias veces...
	esto se soluciona, colocandole una bandera
	que indique si ya ha sido dibujada o no.
*/
    public void permitirDibujar(){
        nodoArista i = primera;
        for(int k=0; k<numAristas; k++){
            i.arista.dibujada = false;
            i = i.siguiente;
        }
    }
    float[] generarCoeficientesNormal(){
		float[] tmp = {0f, 0f, 0f, 0f};
		Punto3d Vect[] = new Punto3d[2], perp;
		Arista ar;
		boolean orden;
		nodoArista i = primera;
		ar = i.arista;
		orden = i.direccion;
		for(int j=0; j<2; j++){
			ar = i.arista;
			orden = i.direccion;
			Vect[j] = (orden)?
				new Punto3d(ar.v2.x-ar.v1.x, ar.v2.y-ar.v1.y, ar.v2.z-ar.v1.z):
				new Punto3d(ar.v2.x-ar.v1.x, ar.v2.y-ar.v1.y, ar.v2.z-ar.v1.z);
			i = i.siguiente;
		}
		perp = Vect[0].cruz(Vect[1]);
		tmp[0] = perp.x;
		tmp[1] = perp.y;
		tmp[2] = perp.z;
		tmp[3] -= perp.x * ar.v1.x + perp.y * ar.v1.y + perp.z * ar.v1.z;
		return tmp;
	}
/*	devuelve el vrtice ms cercano al plano de visin
	en esta lista de aristas.
*/
    public Vertice VerticeMasCercano(){
		nodoArista
			nacab, nacol,	//mnimos de cabeza y cola
			ni = primera;
		float zmccab, zmccol;	//z ms cercano... de la cabeza y de la cola

		nacab = primera;
		zmccab = nacab.cabeza().zp;
		for(int i=1; i<numAristas; i++, ni = ni.siguiente)
			if(zmccab > ni.cabeza().zp){
				zmccab = ni.cabeza().zp;
				nacab = ni;
			}

		nacol = primera;
		zmccol = nacab.cola().zp;

		for(int i=1; i<numAristas; i++, ni = ni.siguiente)
			if(zmccol > ni.cabeza().zp){
				zmccol = ni.cabeza().zp;
				nacol = ni;
			}

		return (zmccab > zmccol)?nacol.cola():nacab.cabeza();
	}

/*	Calcula el promedio de todas las coordenadas zp
	de esta lista de aristas.
	* Se asume que esta lista de aristas es tal,
	* que el vrtice cabeza de una arista
	* es el cola del siguiente, y adems,
	* que la primera arista conecta con la ltima.
*/
	public float promedioZ(){
		float peso = 1f/numAristas, pro=0f;
		nodoArista na = primera;
		for(int i=0; i<numAristas; i++, na = na.siguiente)
			pro += peso * na.cabeza().zp;
		return pro;
	}
}
class nodoArista{
    boolean direccion = true;
    nodoArista anterior=null, siguiente = null;
    Arista arista = null;
    public nodoArista(Arista arista){
        this.arista = arista;
        direccion = true;
    }
    public nodoArista(Arista arista, boolean direccion){
        this.arista = arista;
        this.direccion = direccion;
    }
    public Vertice cabeza(){
		return direccion?arista.v2:arista.v1;
	}
	public Vertice cola(){
		return direccion?arista.v1:arista.v2;
	}
}
class Arista{
    int ix=-1;
    Vertice v1=null, v2=null;
    boolean dibujada = false;
    public Arista(Vertice v1, Vertice v2, int indice){
        this.v1 = v1;
        this.v2 = v2;
        this.ix = indice;
    }
}
class Vertice{
    int ix=-1;
    float
        x=0f, y=0f, z=0f,       //en coordenadas mundiales
        xp=0f, yp=0f, zp=0f;    //en coordenadas de visin
//    listaAristas aristas=null;	//al final, no necesitamos estas referencias
    public Vertice(float x, float y, float z, int indice){
        this.x = x;
        this.y = y;
        this.z = z;
        this.ix = indice;
    }
    public Punto3d punto_3d(){
		return new Punto3d(x, y, z);
	}
    public String toString(){
        return xp+","+yp+","+zp+";";
    }
}
class nodoVertice{
    nodoVertice anterior=null, siguiente=null;
    Vertice vertice = null;
    public nodoVertice(Vertice vertice){
        this.vertice = vertice;
    }
}
class listaVertices{
    int ix=-1, numVertices = 0;
    nodoVertice primero=null, vi=null;

    public listaVertices(Vertice[] vertices){
        nodoVertice v;
        this.primero = new nodoVertice(vertices[0]);
        v = this.primero;
        numVertices = vertices.length;

        for(int i=1; i<numVertices; i++){
            v.siguiente = new nodoVertice(vertices[i]);
            v.siguiente.anterior = v;
            v = v.siguiente;
        }
        v.siguiente = this.primero;
        this.primero.anterior = v;
        vi = this.primero;
        ix = 0;
    }
    public listaVertices(){
        numVertices = 0;
        primero = null;
    }
    public void agregar(Vertice v){
        nodoVertice nodo = new nodoVertice(v);
        if(numVertices>0){
            primero.anterior.siguiente = nodo;
            nodo.anterior = primero.anterior;
            nodo.siguiente = primero;
            primero.anterior = nodo;
        }else{
            primero = nodo;
            primero.anterior = primero.siguiente = nodo;
        }
        numVertices++;
    }
    public void agregar(Vertice[] vs){
        int ind;
        nodoVertice[] nv = new nodoVertice[vs.length];
        for(ind = 0; ind < vs.length; ind++)
            nv[ind] = new nodoVertice(vs[ind]);
        for(ind = 1; ind < vs.length-1; ind++){
            nv[ind].anterior = nv[ind-1];
            nv[ind].siguiente = nv[ind+1];
        }
        if(numVertices>0){
            primero.anterior.siguiente = nv[0];
            nv[0].anterior = primero.anterior;
            nv[vs.length-1].siguiente = primero;
            primero.anterior = nv[vs.length-1];
        }else{
            this.primero = nv[0];
            primero.anterior = nv[vs.length-1];
            nv[vs.length-1].siguiente = primero;
        }
        numVertices += vs.length;
    }
    public void transformar(Matriz3d m){
        nodoVertice i = this.primero;
        for(int k=0; k<numVertices; k++, i=i.siguiente){
            m.transformar(i.vertice);
        }
    }
}
