package proyectoGPC;
/**
 * Contiene definiciones de algunos objetos
 *
 * Una simulacin de un cubo, pir�ide y otras cosas.
 *
 * @author (Eduardo Navas)
 * @version (0.1)
 */
public class Objetos3D{
    public static String[] tipos =
    	{"cubo", "piramide", "cubo_hoyo", "toro", "copa", "paraboloide", "moebius"};
    //	base de figuras (datos) para el paquete
    public static ObjetoEnmallado[] figuras = new ObjetoEnmallado[tipos.length];

    //	inicializador para la base de figuras
    static {
		for(int i = 0; i<tipos.length; i++)
			figuras[i] = Figura(tipos[i]);
	}

	//	para centralizar la creacin de los objetos
    public static ObjetoEnmallado Figura(String tipo){
        if(tipo.equals(tipos[0]))
            return cubo(75);
        else
        if(tipo.equals(tipos[1]))
            return piramide(75);
        else
        if(tipo.equals(tipos[2]))
            return cubo_hoyo(100);
        else
        if(tipo.equals(tipos[3]))
            return toro4(50, 30);
        else
        if(tipo.equals(tipos[4]))
        	return copa(75f, 75f, 20f, 175f, 50f, 30, 80);
        else
        if(tipo.equals(tipos[5]))
        	return paraboloide(200f, 50);
        else
        if(tipo.equals(tipos[6]))
            return moebius(8, 180, 20, 150);//15,80
        else
            return null;
    }

	//	Cinta de Moebius
	public static ObjetoEnmallado moebius(
			float anchoCinta, float radioPrincipal,
			int numDiscos, int numPuntosporDisco){

		final int Tv = numDiscos, Tu = numPuntosporDisco;
		final float li = -anchoCinta/2, ls = anchoCinta/2; //l�ites inferior y superior

		int i, j, contA=0;
		double h, u, v;	//auxiliares

		Vertice[][] vert = new Vertice[Tu][Tv];
		Arista[][] av = new Arista[Tu][Tv-1], ah = new Arista[Tu][Tv];

        listaVertices lv = new listaVertices();
        listaAristas la = new listaAristas();
        listaPoligonos lp = new listaPoligonos();

		//	calculando los puntos y agrup�dolos de una vez
		for(j=0; j<Tv; j++){
			v = j*(ls-li)/(Tv-1) + li;
			for(i=0; i<Tu; i++){
				u = 2*Math.PI*i/Tu;
				h = anchoCinta*v*Math.sin(u/2)+radioPrincipal;
				lv.agregar(
					vert[i][j] = new Vertice(
						(float)(h*Math.cos(u)),
						(float)(h*Math.sin(u)),
						(float)(anchoCinta*v*Math.cos(u/2)), j*Tv+i)
				);
			}
		}

		//	construyendo las aristas... verticales
		for(i=0; i<Tu; i++)
			for(j=0; j<Tv-1; j++)
				av[i][j] = new Arista(vert[i][j], vert[i][j+1], contA++);

		//	... y las horizontales
		for(i=0; i<Tu-1; i++)
			for(j=0; j<Tv; j++)
				ah[i][j] = new Arista(vert[i][j], vert[i+1][j], contA++);
		//conexión del doblés
		for(j=0;j<Tv;j++)
			ah[Tu-1][j]=new Arista(vert[Tu-1][j], vert[0][Tv-j-1], contA++);
		//	generando los polígonos
		for(i=0; i<Tu-1; i++)
			for(j=0; j<Tv-1; j++)
				lp.agregar(new Arista[]{
					av[i][j], ah[i][j+1],
					av[i+1][j], ah[i][j]
				}, new boolean[]{true,true,false,false});
        
        for(j=0; j<Tv-1; j++)
            lp.agregar(new Arista[]{
                av[Tu-1][j], ah[Tu-1][j+1],
                av[0][Tv-j-2], ah[Tu-1][j]
            }, new boolean[]{true, true, true, false});

		//	agrupando las aristas... en cualquier orden
		for(j=0; j<Tu; j++)
	        for(i=0; i<Tv-1; i++)
				la.agregar(av[j][i]);
		for(j=0; j<Tv; j++)
	        for(i=0; i<Tu-1; i++)
    			la.agregar(ah[i][j]);

		return new ObjetoEnmallado(lp, la, lv, (float)(radioPrincipal+2*ls));
	}

	//	la definicin de un paraboloide
    public static ObjetoEnmallado paraboloide(float ancho, int numPuntos){
		Vertice[][] vert = new Vertice[numPuntos][numPuntos];
		Arista[][]
			av = new Arista[numPuntos][numPuntos-1],
			ah = new Arista[numPuntos-1][numPuntos];
        listaVertices lv = new listaVertices();
        listaAristas la = new listaAristas();
        listaPoligonos lp = new listaPoligonos();

		int i, j, contA=0;
		float lim = -ancho/2;
		float x, y;

		//	calcular puntos
		for(i=0; i<numPuntos; i++){
			x = 2*lim*i/(numPuntos-1)-lim;
			for(j=0; j<numPuntos; j++){
				y = 2*lim*j/(numPuntos-1)-lim;
				lv.agregar(vert[i][j] = new Vertice(x, y,
//					(float)(Math.sqrt(40000-x*x - y*y)),	//funcin esfera
//					(float)(Math.sqrt(x*x + y*y)),	//funcin cono
					(float)((x*x + y*y)/100),	//funcin paraboloide
				i*numPuntos+j));
			}
		}
		//	costruir aristas y registrarlas de una vez
		for(i=0; i<numPuntos; i++){
			for(j=0; j<numPuntos-1; j++){
				av[i][j] = new Arista(vert[i][j], vert[i][j+1], contA++);
			}
		}
		for(j=0; j<numPuntos; j++){
			for(i=0; i<numPuntos-1; i++){
				ah[i][j] = new Arista(vert[i][j], vert[i+1][j], contA++);
			}
		}
		//	construir pol�onos
		for(i=0; i<numPuntos-1; i++){
			for(j=0; j<numPuntos-1; j++){
				lp.agregar(
					new Arista[]{ah[i][j], av[i+1][j], ah[i][j+1], av[i][j]},
					new boolean[]{false,false,true,true});
			}
		}
		//	registrar las aristas
		for(i=0; i<numPuntos; i++){
	        for(j=0; j<numPuntos-1; j++){
				la.agregar(av[i][j]);
			}
        }
		for(j=0; j<numPuntos; j++){
	        for(i=0; i<numPuntos-1; i++){
				la.agregar(ah[i][j]);
			}
        }
		return new ObjetoEnmallado(lp, la, lv, 205f);
	}

	//	parametrizacin de una copa (con cosenos)
	public static ObjetoEnmallado copa(
			float AmplitudCopa, float AmplitudBase,
			float Grosor, float maxCopa, float maxBase,
			int numDiscos, int numPuntosporDisco){

		float lambda = (float)(Math.PI / maxBase), omega = (float)(Math.PI / maxCopa);
		float Alambda = AmplitudBase, Aomega = AmplitudCopa, G = Grosor;
		final int Tu = numDiscos, Tv = numPuntosporDisco;
		final float li = -maxBase, ls = maxCopa; //l�ites inferior y superior

		int i, j, k, contA = 0;
		double	h, u, v;	//auxiliares

		Vertice[][] vert = new Vertice[Tu][Tv];
		Arista[][] av = new Arista[Tv][Tu-1], ah = new Arista[Tu][Tv];

        listaVertices lv = new listaVertices();
        listaAristas la = new listaAristas();
        listaPoligonos lp = new listaPoligonos();

		//	calculando los puntos y agrup�dolos de una vez
		for(i=0; i<Tu; i++){
			u = i*(ls-li)/(Tu-1) + li;
			h = (u<0.0)?
				-Alambda*Math.cos(lambda*u)+Alambda+G:
				-Aomega *Math.cos(omega *u)+Aomega +G;
			for(j=0; j<Tv; j++){
				v = 2*Math.PI*j/Tv;
				lv.agregar(
					vert[i][j] = new Vertice(
						(float) (h*Math.cos(v)),
						(float) (h*Math.sin(v)),
						(float)u, i*Tu+j)
				);
			}
		}

		//	construyendo las aristas... verticales
		for(j=0; j<Tv; j++)
		for(i=0; i<Tu-1; i++)
			av[j][i] = new Arista(vert[i][j], vert[i+1][j], contA++);
		//	... y las horizontales
		for(i=0; i<Tu; i++){
			for(j=0; j<Tv-1; j++)
				ah[i][j] = new Arista(vert[i][j], vert[i][j+1], contA++);
			ah[i][Tv-1] = new Arista(vert[i][Tv-1], vert[i][0], contA++);
		}

		//	generando los pol�onos
		for(i=0; i<Tu-1; i++){
			for(j=0; j<Tv-1; j++)
				lp.agregar(new Arista[]{
					av[j][i], ah[i+1][j],
					av[j+1][i], ah[i][j]
				}, new boolean[]{true,true,false,false});
			lp.agregar(new Arista[]{
					av[Tv-1][i],ah[i+1][Tv-1],
					av[0][i], ah[i][Tv-1]
				}, new boolean[]{true,true,false,false});
		}

		//	agrupando las aristas... en cualquier orden
		for(j=0; j<Tv; j++){
	        for(i=0; i<Tu-1; i++){
				la.agregar(av[j][i]);
				la.agregar(ah[i][j]);
			}
			la.agregar(ah[Tu-1][j]);
        }

		return new ObjetoEnmallado(lp, la, lv, (float)Math.sqrt(
				Math.pow(Math.max(maxBase, maxCopa),2)+
				Math.pow(2*Math.max(Alambda, Aomega)+G,2)
		));
	}

	//	un toro...
    public static ObjetoEnmallado toro4(int numDiscos, int numPuntosporDisco){
        float mag;

        int tu = numDiscos, tv = numPuntosporDisco;
        float a = 120f, b = 60f;
        int i, j, contA = 0;
        Vertice[][] v = new Vertice[tu][tv];
        Arista[][] av = new Arista[tu][tv], ah = new Arista[tv][tu];

        listaVertices lv = new listaVertices();
        listaAristas la = new listaAristas();
        listaPoligonos lp = new listaPoligonos();

        //	calculando los puntos
        for(i = 0; i < tu; i++)
        for(j = 0; j < tv; j++)
            v[i][j] = new Vertice(
                (float)((a + b*Math.cos(2*Math.PI*j/tv))*Math.cos(2*Math.PI*i/tu)),
                (float)((a + b*Math.cos(2*Math.PI*j/tv))*Math.sin(2*Math.PI*i/tu)),
                (float)(b*Math.sin(2*Math.PI*j/tv)),
                i*tu+j
            );

		//	punto m� lejano posible:
		mag = (float) Math.sqrt(Math.pow(
			a + b
		,2)+Math.pow(
			a + b
		,2)+Math.pow(
			b
		,2));

		//	construyendo las aristas
        for(i=0; i<tu; i++){
            for(j=0; j<tv-1; j++)
                av[i][j] = new Arista(v[i][j], v[i][j+1], contA++);
            av[i][tv-1] = new Arista(v[i][tv-1], v[i][0], contA++);
        }
        for(j=0; j<tv; j++){
            for(i=0; i<tu-1; i++)
                ah[j][i] = new Arista(v[i][j], v[i+1][j], contA++);
            ah[j][tu-1] = new Arista(v[tu-1][j], v[0][j], contA++);
        }

		//	generando los poligonos
        for(i = 0; i<tu-1; i++){
            for(j=0; j<tv-1; j++)
                lp.agregar(new Arista[]{
                    av[i][j], ah[j][i],
                    av[i+1][j], ah[j+1][i]
                }, new boolean[]{
					false, true, true, false
				});
            lp.agregar(new Arista[]{
                av[i][tv-1], ah[tv-1][i],
                av[i+1][tv-1], ah[0][i]
            }, new boolean[]{
				false, true, true, false
			});
        }
        for(j=0; j<tv-1; j++)
            lp.agregar(new Arista[]{
                av[tu-1][j], ah[j][tu-1],
                av[0][j], ah[j+1][tu-1]
            }, new boolean[]{
				false, true, true, false
			});
        lp.agregar(new Arista[]{
            av[tu-1][tv-1], ah[tv-1][tu-1],
            av[0][tv-1], ah[0][tu-1]
        }, new boolean[]{
			false, true, true, false
		});

		//	agrupando los v�tices
        for(i=0; i<tu; i++)
        for(j=0; j<tv; j++)
            lv.agregar(v[i][j]);

		//	agrupando las aristas
        for(i=0; i<tu; i++)
        for(j=0; j<tv; j++){
            la.agregar(av[i][j]);
            la.agregar(ah[j][i]);
        }
        return new ObjetoEnmallado(lp, la, lv, mag);
    }

    //	una pir�ide sencilla
    public static ObjetoEnmallado piramide(float tam){
		float mag;

        Vertice[] v;
        Arista[] a;
        listaAristas[] p;

        v = new Vertice[5];
        v[0] = new Vertice(   0,   0,2*tam, 0);
        v[1] = new Vertice(-tam, tam,  0, 1);
        v[2] = new Vertice( tam, tam,  0, 2);
        v[3] = new Vertice( tam,-tam,  0, 3);
        v[4] = new Vertice(-tam,-tam,  0, 4);

		mag = 2*tam;

        a = new Arista[8];
        a[0] = new Arista(v[0], v[1], 0);
        a[1] = new Arista(v[0], v[2], 1);
        a[2] = new Arista(v[0], v[3], 2);
        a[3] = new Arista(v[0], v[4], 3);
        a[4] = new Arista(v[1], v[2], 4);
        a[5] = new Arista(v[2], v[3], 5);
        a[6] = new Arista(v[3], v[4], 6);
        a[7] = new Arista(v[4], v[1], 7);

        p = new listaAristas[5];
        p[0] = new listaAristas(new Arista[]{a[0], a[4], a[1]}, new boolean[]{true,true,false});
        p[1] = new listaAristas(new Arista[]{a[1], a[5], a[2]}, new boolean[]{true,true,false});
        p[2] = new listaAristas(new Arista[]{a[2], a[6], a[3]}, new boolean[]{true,true,false});
        p[3] = new listaAristas(new Arista[]{a[3], a[7], a[0]}, new boolean[]{true,true,false});
        p[4] = new listaAristas(new Arista[]{a[4], a[7], a[6], a[5]}, new boolean[]{false,false,false,false});

        return new ObjetoEnmallado(p, a, v, mag);
    }

    //	un vulgar hexahedro
    public static ObjetoEnmallado cubo(float tam){
		float mag;

        Vertice[] v;
        Arista[] a;
        listaAristas[] p;

        v = new Vertice[8];
        v[0] = new Vertice(-tam, -tam, -tam, 0);
        v[1] = new Vertice(-tam, -tam,  tam, 1);
        v[2] = new Vertice(-tam,  tam, -tam, 2);
        v[3] = new Vertice(-tam,  tam,  tam, 3);
        v[4] = new Vertice( tam, -tam, -tam, 4);
        v[5] = new Vertice( tam, -tam,  tam, 5);
        v[6] = new Vertice( tam,  tam, -tam, 6);
        v[7] = new Vertice( tam,  tam,  tam, 7);

		mag = (float)(tam * Math.sqrt(3));

        a = new Arista[12];
        a[0] = new Arista(v[0], v[1], 0);
        a[1] = new Arista(v[1], v[3], 1);
        a[2] = new Arista(v[3], v[2], 2);
        a[3] = new Arista(v[2], v[0], 3);
        a[4] = new Arista(v[4], v[5], 4);
        a[5] = new Arista(v[5], v[7], 5);
        a[6] = new Arista(v[7], v[6], 6);
        a[7] = new Arista(v[6], v[4], 7);
        a[8] = new Arista(v[1], v[5], 8);
        a[9] = new Arista(v[0], v[4], 9);
        a[10] = new Arista(v[3], v[7], 10);
        a[11] = new Arista(v[2], v[6], 11);

        p = new listaAristas[6];
        p[0] = new listaAristas(new Arista[]{a[3], a[2], a[1], a[0]}, new boolean[]{false,false,false,false});
        p[1] = new listaAristas(new Arista[]{a[0], a[8], a[4], a[9]}, new boolean[]{true,true,false,false});
        p[2] = new listaAristas(new Arista[]{a[4], a[5], a[6], a[7]}, new boolean[]{true,true,true,true});
        p[3] = new listaAristas(new Arista[]{a[6], a[10], a[2], a[11]}, new boolean[]{false,false,true,true});
        p[4] = new listaAristas(new Arista[]{a[3], a[9], a[7], a[11]}, new boolean[]{true,true,false,false});
        p[5] = new listaAristas(new Arista[]{a[1], a[8], a[5], a[10]}, new boolean[]{false,true,true,false});

        return new ObjetoEnmallado(p, a, v, mag);
    }

    //	un cubo con un cubo adentro... no muy bien definido, pero se entiende
    public static ObjetoEnmallado cubo_hoyo(float tam){
		float mag;

        Vertice[] v;
        Arista[] a;
        listaAristas[] p;

        v = new Vertice[16];
        v[0] = new Vertice(-tam, -tam, -tam, 0);
        v[1] = new Vertice(-tam, -tam,  tam, 1);
        v[2] = new Vertice(-tam,  tam, -tam, 2);
        v[3] = new Vertice(-tam,  tam,  tam, 3);
        v[4] = new Vertice( tam, -tam, -tam, 4);
        v[5] = new Vertice( tam, -tam,  tam, 5);
        v[6] = new Vertice( tam,  tam, -tam, 6);
        v[7] = new Vertice( tam,  tam,  tam, 7);

        v[ 8] = new Vertice(-tam/2, -tam/2,  0, 8);
        v[ 9] = new Vertice(-tam/2, -tam/2,tam, 9);
        v[10] = new Vertice(-tam/2,  tam/2,  0,10);
        v[11] = new Vertice(-tam/2,  tam/2,tam,11);
        v[12] = new Vertice( tam/2, -tam/2,  0,12);
        v[13] = new Vertice( tam/2, -tam/2,tam,13);
        v[14] = new Vertice( tam/2,  tam/2,  0,14);
        v[15] = new Vertice( tam/2,  tam/2,tam,15);

		mag = (float)(tam * Math.sqrt(3));

        a = new Arista[24];
        a[0] = new Arista(v[0], v[1], 0);
        a[1] = new Arista(v[1], v[3], 1);
        a[2] = new Arista(v[3], v[2], 2);
        a[3] = new Arista(v[2], v[0], 3);
        a[4] = new Arista(v[4], v[5], 4);
        a[5] = new Arista(v[5], v[7], 5);
        a[6] = new Arista(v[7], v[6], 6);
        a[7] = new Arista(v[6], v[4], 7);
        a[8] = new Arista(v[1], v[5], 8);
        a[9] = new Arista(v[0], v[4], 9);
        a[10] = new Arista(v[3], v[7], 10);
        a[11] = new Arista(v[2], v[6], 11);

        a[12] = new Arista(v[8], v[9],12);
        a[13] = new Arista(v[9], v[11],13);
        a[14] = new Arista(v[11], v[10],14);
        a[15] = new Arista(v[10], v[8],15);
        a[16] = new Arista(v[12], v[13],16);
        a[17] = new Arista(v[13], v[15],17);
        a[18] = new Arista(v[15], v[14],18);
        a[19] = new Arista(v[14], v[12],19);
        a[20] = new Arista(v[9], v[13],20);
        a[21] = new Arista(v[8], v[12],21);
        a[22] = new Arista(v[11], v[15],22);
        a[23] = new Arista(v[10], v[14],23);

        p = new listaAristas[12];
        p[0] = new listaAristas(new Arista[]{a[3], a[2], a[1], a[0]}, new boolean[]{false,false,false,false});
        p[1] = new listaAristas(new Arista[]{a[0], a[8], a[4], a[9]}, new boolean[]{true,true,false,false});
        p[2] = new listaAristas(new Arista[]{a[4], a[5], a[6], a[7]}, new boolean[]{true,true,true,true});
        p[3] = new listaAristas(new Arista[]{a[6], a[10], a[2], a[11]}, new boolean[]{false,false,true,true});
        p[4] = new listaAristas(new Arista[]{a[3], a[9], a[7], a[11]}, new boolean[]{true,true,false,false});
        p[5] = new listaAristas(new Arista[]{a[1], a[8], a[5], a[10]}, new boolean[]{false,true,true,false});

        p[6] = new listaAristas(new Arista[]{a[15], a[14], a[13], a[12]}, new boolean[]{false,false,false,false});
        p[7] = new listaAristas(new Arista[]{a[12], a[20], a[16], a[21]}, new boolean[]{true,true,false,false});
        p[8] = new listaAristas(new Arista[]{a[16], a[17], a[18], a[19]}, new boolean[]{true,true,true,true});
        p[9] = new listaAristas(new Arista[]{a[18], a[22], a[14], a[23]}, new boolean[]{false,false,true,true});
        p[10] = new listaAristas(new Arista[]{a[15], a[21], a[19], a[23]}, new boolean[]{true,true,false,false});
        p[11] = new listaAristas(new Arista[]{a[13], a[20], a[17], a[22]}, new boolean[]{false,true,true,false});

        return new ObjetoEnmallado(p, a, v, mag);
    }
}	//fin de la clase Objetos3D
